package com.chinacreator.video.aspect;
import javax.servlet.http.HttpServletRequest;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import com.alibaba.fastjson.JSON;
import com.chinacreator.video.dto.DataOutPipe;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.log.HandleLog;
import com.chinacreator.video.util.IpAddrUtil;
import com.chinacreator.video.util.URLUtil;

@Aspect
@Component
public class LogAspect {

	private static Logger logger=LoggerFactory.getLogger("INFO");  
	
	
	@Pointcut("@annotation(com.chinacreator.video.annotation.Log)")
    public void webLog(){}

	@Around("webLog()")
	public Object showLog(ProceedingJoinPoint point) throws Throwable {
		long startTime = System.currentTimeMillis();
		HandleLog handleLog = new HandleLog();
		Object[] args = point.getArgs();
		ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
		HttpServletRequest request = attributes.getRequest();
		Signature signature = point.getSignature();
		String name = signature.getName();
		handleLog.setHostip(request.getServerName());
		handleLog.setHostport(String.valueOf(request.getServerPort()));
		handleLog.setAction(name);
		handleLog.setIp(IpAddrUtil.getIpAddr(request));
		try {
			if(name.equals("asynOrder")) {
				handleLog.setReq(URLUtil.readInputStream(request.getInputStream()));
			}else {
				handleLog.setReq(JSON.toJSONString(args));
			}
		}catch(Exception e) {
		}
		Object result = null;
		try {
			if(name.equals("asynOrder")) {
				result = point.proceed(new Object[] {handleLog.getReq()});
			}else {
				result = point.proceed();
			}
			handleLog.setResultcode("0");
			handleLog.setErrorinfo("");
			handleLog.setRsp(JSON.toJSONString(result));
		} catch (Exception e) {
			DataOutPipe<?> r = new DataOutPipe<>();
			if(e instanceof BussinessException) {
				 r.setErrorinfo(e.getMessage());
			     r.setResultcode(((BussinessException) e).getCode());
			}else {
				r.setErrorinfo("系统忙，请稍后再试");
		        r.setResultcode("8000");
			}
			handleLog.setResultcode(r.getResultcode());
			handleLog.setErrorinfo(r.getErrorinfo());
			handleLog.setRsp(JSON.toJSONString(r));
			throw e;
		}finally {
			handleLog.setTimes((System.currentTimeMillis()-startTime)+"");
			logger.info(JSON.toJSONString(handleLog));
		}
		return result;
	}
}
