package com.chinacreator.video.service;


import com.alibaba.fastjson.JSON;
import com.chinacreator.video.bean.ActivityGiftBean;
import com.chinacreator.video.bean.ActivityLimitConfig;
import com.chinacreator.video.bean.UserPrizeBean;
import com.chinacreator.video.bean.VipInfo;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.mapper.ActiveAiqy20AugustDao;
import com.chinacreator.video.mapper.ActiveAiqy20Dao;
import com.chinacreator.video.mapper.RaffleActivityConfigDao;
import com.chinacreator.video.redis.RedisService;
import com.chinacreator.video.util.IpAddrUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

@Component
public class ActivityAiqy20AugustService {

    private static Logger ErrorLOGGER = LoggerFactory.getLogger("ERROR");

    @Autowired
    private RedisService redisService;

    @Autowired
    private ComonService comonService;

    @Autowired
    private MobService mobService;
    // 放大倍数
    private static final int mulriple = 10000;


    @Autowired
    private RaffleActivityConfigDao configDao;

    @Autowired
    private ActiveAiqy20AugustDao activeDao;

    /**
     * 取值
     * @param key
     * @return
     * @throws BussinessException
     */
    public long getRedis(String key) throws BussinessException {
        try {
            String count = redisService.get(key);
            System.out.println(key+":"+count);
            if(StringUtils.isEmpty(count)){
                return 0;
            }
            return Long.parseLong(count);

        } catch (Exception e) {

            ErrorLOGGER.error("", e);

        }
        return 0;
    }


    /**
     * 设值
     * @param key
     * @param value
     * @throws BussinessException
     */
    public void setRedis(String key,String value) throws BussinessException {
        try {
            redisService.add(key,value);
            System.out.println(key+":"+value);
        } catch (Exception e) {
            ErrorLOGGER.error("", e);
        }
    }

    /**
     *  加1
     * @param key
     * @return
     * @throws BussinessException
     */
    public long addRedis(String key) throws BussinessException {
        try {
            long count = redisService.incrAdd(key,60L*60L*24L*100L);
            System.out.println(key+":"+count);
            return count;
        } catch (Exception e) {
            ErrorLOGGER.error("", e);
        }
        return 0;
    }

    /**
     * 减1
     * @param key
     * @return
     * @throws BussinessException
     */
    public long backRedis(String key) throws BussinessException{
        try {
            long count = redisService.incrBack(key,60L*60L*24L*100L);
            System.out.println(key+":"+count);
            return count;
        } catch (Exception e) {
            ErrorLOGGER.error("", e);
        }
        return 0;

    }




    /**
     * 查询检验
     * @param vipInfo
     * @param config
     * @throws BussinessException
     */
    public void checkVipQuery(VipInfo vipInfo, ActivityLimitConfig config) throws BussinessException {
        //判断活动是否在有效期内
        this.checkAiqy20(vipInfo,config);

        comonService.checkTimestamp(vipInfo.getTimestamp());

        comonService.checkQueryVipSign(vipInfo);
        mobService.checkVipMob(vipInfo);
    }
    public void checkVipQuery(VipInfo vipInfo) throws BussinessException {
        this.comonService.checkTimestamp(vipInfo.getTimestamp());
        this.comonService.checkQueryVipSign(vipInfo);
        this.mobService.checkVipMob(vipInfo);
    }
    public void checkVipQuery(UserPrizeBean vipInfo) throws BussinessException {
        this.checkRedis(vipInfo);
        this.comonService.checkTimestamp(vipInfo.getTimestamp());
        this.comonService.checkQueryVipSign(vipInfo);
        this.mobService.checkVipMob(vipInfo);
    }
    /**
     * 抽奖校验
     *
     * @param vipInfo
     * @throws BussinessException
     */
    public void checkVip(VipInfo vipInfo,ActivityLimitConfig config) throws BussinessException {
        this.checkRedis(vipInfo);
        this.checkAiqy20(vipInfo, config);
        comonService.checkTimestamp(vipInfo.getTimestamp());
        comonService.checkQueryVipSign(vipInfo);
        mobService.checkVipMob(vipInfo);
    }


    /**
     * 活动抽奖
     *
     * @param activityInfo
     * @throws BussinessException
     */
    public String givePrize(VipInfo activityInfo,ActivityLimitConfig config) throws BussinessException {
        String result="";
        this.queryVip(activityInfo);

        if("aiqy".equals(activityInfo.getCpid()) && "1018".equals(activityInfo.getSpid())){

            result = this.joinPrize(activityInfo, config);


        }else{
            throw new BussinessException("9088", "该cp暂未开放");
        }
        System.out.println("JoinInfo="+ JSON.toJSONString(activityInfo));
        return result;
    }


    /**
     *  抽奖
     * @param activityInfo
     * @param config
     * @return
     */
    public String joinPrize(VipInfo activityInfo,ActivityLimitConfig config)throws BussinessException{
        int count=config.getLimitcount();
        //是否已经领取
        int joincount = activeDao.getJoinActive(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype());
        if(joincount > 0) {
            throw new BussinessException("6081", "已参加活动");
        }

        String province = activeDao.getOrderPrivince(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid());
        long vipcount2 = this.getRedis(config.getRediskey());
        if (vipcount2 >= count) {
            activeDao.insert(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype(),  province,"未中奖",  "5");
            throw new BussinessException("6083", "未中奖");
        }
        int vipcount=activeDao.getCount();
        if (vipcount >= count) {
            activeDao.insert(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype(),  province,"未中奖",  "5");
            throw new BussinessException("6083", "未中奖");
        }
        int redid=0;

        List<ActivityGiftBean> prizes =configDao.getGiftConfig();

        //领取红包
        //1.查询配置文件红包金额计算比例
        try {
            for(int i=0;i<prizes.size();i++){
                ActivityGiftBean bean = prizes.get(i);
                long usecount = this.getRedis(config.getRediskey() + bean.getId());
                int prizecount = bean.getCount();

                if (usecount >= prizecount) {
                    prizes.remove(bean);
                    i--;

                } else {

                    prizecount = prizecount - (int) usecount;
                    bean.setRealcount(prizecount);
                    if (vipcount2 < count) {
                        int realcount = (int) (count - vipcount2);
                        float nerpercent = (float) prizecount / realcount;
                        BigDecimal f1 = new BigDecimal(nerpercent);
                        try {
                            bean.setPercent(f1.setScale(4, BigDecimal.ROUND_HALF_UP));

                        } catch (Exception e) {
                            ErrorLOGGER.info(e.getMessage() + "总数" + prizes + count + "百分比" + f1);
                        }
                    }
                }
            }
        }catch (Exception e){
            throw new BussinessException("8000","计算异常"+e.getMessage());
        }

        if(prizes.size()==0 || prizes.isEmpty()){
            activeDao.insert(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype(),  province,"未中奖",  "5");
            throw new BussinessException("6083", "未中奖");
        }
        redid=this.pay(prizes);
        if(redid>0){
            for(int i=0;i<prizes.size();i++){
                ActivityGiftBean bean=prizes.get(i);
                if(bean.getId()==redid){
                    long usecount= this.addRedis(config.getRediskey()+bean.getId());
                    System.out.println("使用id"+redid+usecount);
                    if(usecount>bean.getCount()){
                        activeDao.insert(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype(),  province,"未中奖",  "5");
                        throw new BussinessException("6083", "未中奖");
                    }
                    try {
                        //保存领取日志
                        String id = new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date()) + String.valueOf((int) ((Math.random() * 9 + 1) * 1000));
                        activeDao.insert(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype(),  province,bean.getPrize(), bean.getId() + "");
                        //redis总数计数
                        this.addRedis(config.getRediskey());
                        return bean.getId()+"";
                    }catch (Exception e){
                        ErrorLOGGER.info(e.getMessage());
                        this.backRedis(config.getRediskey()+bean.getId());
                        this.backRedis(config.getRediskey());
                        activeDao.insert(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype(),  province,"未中奖",  "5");
                        throw new BussinessException("6083", "未中奖");
                    }
                }
            }

        }else{
            activeDao.insert(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype(),  province,"未中奖",  "5");
            throw new BussinessException("6083", "未中奖");
        }

        activeDao.insert(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype(),  province,"未中奖",  "5");
        throw new BussinessException("6083", "未中奖");
    }


    /**
     * 随机抽奖
     * @param prizes
     * @return
     */
    public int pay(List<ActivityGiftBean> prizes) {
        int lastScope = 0;
        // 洗牌，打乱奖品次序
        Collections.shuffle(prizes);
        Map<Integer, int[]> prizeScopes = new HashMap<Integer, int[]>();
        Map<Integer, Integer> prizeQuantity = new HashMap<Integer, Integer>();
        for (ActivityGiftBean prize : prizes) {
            int prizeId = prize.getId();
            // 划分区间
            int currentScope = lastScope + prize.getPercent().multiply(new BigDecimal(mulriple)).intValue();
            prizeScopes.put(prizeId, new int[] { lastScope + 1, currentScope });
            prizeQuantity.put(prizeId, prize.getRealcount());

            lastScope = currentScope;
        }

        // 获取1-1000000之间的一个随机数
        int luckyNumber = new Random().nextInt(mulriple);
        if(luckyNumber==0){
            luckyNumber++;
        }
        int luckyPrizeId = 0;
        // 查找随机数所在的区间
        if ((null != prizeScopes) && !prizeScopes.isEmpty()) {
            Set<Map.Entry<Integer, int[]>> entrySets = prizeScopes.entrySet();
            for (Map.Entry<Integer, int[]> m : entrySets) {
                int key = m.getKey();
                if (luckyNumber >= m.getValue()[0] && luckyNumber <= m.getValue()[1] && prizeQuantity.get(key) > 0) {
                    luckyPrizeId = key;
                    break;
                }
            }
        }
        return luckyPrizeId;
    }





    /**
     * 检查是否是爱奇艺抽奖活动且有效
     * @param vipInfo
     * @throws BussinessException
     */
    public void checkAiqy20(VipInfo vipInfo,ActivityLimitConfig config) throws BussinessException {
        String activetype=config.getActivetype();
        String activeend=config.getActiveendtime();

        if(!activetype.equals(vipInfo.getActivetype())){
            throw new BussinessException("9083", "该活动无效");
        }

        try {
            if(new SimpleDateFormat("yyyyMMddHHmmss").parse(activeend).getTime()<System.currentTimeMillis()){
                throw new BussinessException("9083", "该活动已经结束");
            }
        } catch (Exception e) {
            if (e instanceof BussinessException) {
                throw (BussinessException)e;
            }else{
                ErrorLOGGER.error("", e);
            }
        }
    }


    /**
     *
     * @param activityInfo
     * @throws BussinessException
     */
    public void queryVip(VipInfo activityInfo) throws BussinessException {
        //是否已经抽奖
        int joincount = activeDao.getJoinActive(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype());
        if(joincount > 0) {
//            if("18673197465".equals(activityInfo.getUserid())){

//            }else{

                throw new BussinessException("6081", "已参与抽奖");
//            }
        }

        //是否有效订购
        int ordercount = activeDao.getOrderRelation(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid());
        if(ordercount<=0){
            throw new BussinessException("6082", "用户不具备抽奖资格");
        }



    }

    public String queryPrize(VipInfo activityInfo) throws BussinessException {
        int joincount = this.activeDao.getrealJoinActive(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype());
        if (joincount == 0) {
            throw new BussinessException("6083", "您未参与此次抽奖哦！");
        } else {
            String prize = this.activeDao.getPrize(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype());
            if ("未中奖".equals(prize) || "未中奖,未中奖".equals(prize)) {
                throw new BussinessException("6085", "您未中奖哦！");
            } else {
                return prize;
            }
        }
    }

    public void savePrize(UserPrizeBean activityInfo) throws BussinessException {
        int joincount = this.activeDao.getrealJoinActive(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype());
        if (joincount == 0) {
            throw new BussinessException("6083", "您未参与此次抽奖哦！");
        } else {
            String resultcode;
            try {
                int count = this.activeDao.getSavePrize(activityInfo.getUserid());
                if (count > 0) {
                    throw new BussinessException("6086", "您已登记中奖信息！");
                } else {
                    resultcode = this.activeDao.getPrize(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype());
                    this.activeDao.savePrize(activityInfo.getUserid(), activityInfo.getUsername(), activityInfo.getAddress(), activityInfo.getPhone(), resultcode);
                }
            } catch (Exception var6) {
                var6.printStackTrace();
                if (var6 instanceof BussinessException) {
                    resultcode = ((BussinessException)var6).getCode();
                    String errrorinfo = ((BussinessException)var6).getMessage();
                    throw new BussinessException(resultcode, errrorinfo);
                } else {
                    throw new BussinessException("6084", "保存用户领奖信息异常");
                }
            }
        }
    }



    public void checkRedis(VipInfo vipInfo) throws BussinessException {
        try {
            long count = redisService.incr("joinAugustraffle_" + vipInfo.getActivetype() + "_" + vipInfo.getUserid());
            if (count > 1) {
                throw new BussinessException("8001", "并发请求，请稍后再试");
            }
        } catch (Exception e) {
            if (e instanceof BussinessException) {
                throw e;
            }else{
                ErrorLOGGER.error("", e);
            }
        }
    }



    /**
     * ip校验
     */
    public void checkIp(ActivityLimitConfig config)throws BussinessException {

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        if(org.apache.commons.lang.StringUtils.isNotEmpty(config.getLimitip())) {
            System.out.println("ip============>"+config.getLimitip());
            List<String> list = Arrays.asList(config.getLimitip().split(","));
            System.out.println("listIP:"+list);
            String ip = IpAddrUtil.getIpAddr(request);
            System.out.println("ip:"+ip);
            if(!list.contains(ip)) {
                throw new BussinessException("10009", "ip地址校验失败"+ip);
            }
        }
    }

}
