package com.chinacreator.video.service;


import com.alibaba.fastjson.JSON;
import com.chinacreator.video.bean.*;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.mapper.ActiveMgtvShareDao;
import com.chinacreator.video.mapper.CpSpDao;
import com.chinacreator.video.redis.RedisService;
import com.chinacreator.video.util.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.*;

@Component
public class ActivityMgtvShareService {

    private static Logger ErrorLOGGER = LoggerFactory.getLogger("ERROR");

    @Autowired
    private RedisService redisService;

    @Autowired
    private ComonService comonService;

    @Autowired
    private CpSpDao CpSpDao;

    @Autowired
    private MobService mobService;
    // 放大倍数
    private static final int mulriple = 10000;


    @Autowired
    private ActiveMgtvShareDao activeDao;


    /**
     * 取值
     * @param key
     * @return
     * @throws BussinessException
     */
    public long getRedis(String key) throws BussinessException {
        try {
            String count = redisService.get(key);
            System.out.println(key+":"+count);
            if(StringUtils.isEmpty(count)){
                return 0;
            }
            return Long.parseLong(count);

        } catch (Exception e) {

            ErrorLOGGER.error("", e);

        }
        return 0;
    }


    /**
     * 设值
     * @param key
     * @param value
     * @throws BussinessException
     */
    public void setRedis(String key,String value) throws BussinessException {
        try {
            redisService.add(key,value);
            System.out.println(key+":"+value);
        } catch (Exception e) {
            ErrorLOGGER.error("", e);
        }
    }

    /**
     *  加1
     * @param key
     * @return
     * @throws BussinessException
     */
    public long addRedis(String key)
    {
        try {
            long count = redisService.incrAdd(key,60L*60L*24L*100L);
            System.out.println(key+":"+count);
            return count;
        } catch (Exception e) {
            ErrorLOGGER.error("", e);
        }
        return 0;
    }

    /**
     * 减1
     * @param key
     * @return
     * @throws BussinessException
     */
    public long backRedis(String key) throws BussinessException{
        try {
            long count = redisService.incrBack(key,60L*60L*24L*100L);
            System.out.println(key+":"+count);
            return count;
        } catch (Exception e) {
            ErrorLOGGER.error("", e);
        }
        return 0;

    }


    public void checkSign(VipInfo activityInfo) throws BussinessException {

        CpSpBean cpSpBean = CpSpDao.getCpSp(activityInfo.getCpid(), activityInfo.getSpid());
        String pwd = cpSpBean.getPwd();
        String localSign = MD5.MD5Encode(activityInfo.getUserid()+activityInfo.getActivetype() + pwd + activityInfo.getTimestamp());
        if (!localSign.equals(activityInfo.getSignature())) {
            throw new BussinessException("9054", "signature校验失败");
        }
    }

    /**
     * 查询检验
     * @param vipInfo
     * @param config
     * @throws BussinessException
     */
    public void checkVipQuery(VipInfo vipInfo, ActivityLimitConfig config) throws BussinessException {
        //判断活动是否在有效期内
        this.checkCpidspid(vipInfo);
        this.checkAiqy20(vipInfo,config);
        comonService.checkTimestamp(vipInfo.getTimestamp());
        String sign = vipInfo.getSignature();
        if (StringUtils.isEmpty(sign)) {
            throw new BussinessException("9054", "signature不能为空");
        }
        this.checkSign(vipInfo);
        if(!StringUtils.isEmpty(vipInfo.getUserid())){
            mobService.checkVipMob(vipInfo);
        }
       // mobService.checkVipMob(vipInfo);
    }
    //查询获奖记录
    public void checkQueryRanck(VipInfo vipInfo) throws BussinessException {
        this.checkCpidspid(vipInfo);
        this.comonService.checkTimestamp(vipInfo.getTimestamp());
        if (StringUtils.isEmpty(vipInfo.getSignature())) {
            throw new BussinessException("9054", "signature不能为空");
        }
        CpSpBean cpSpBean = CpSpDao.getCpSp(vipInfo.getCpid(), vipInfo.getSpid());
        String pwd = cpSpBean.getPwd();
        String localSign = MD5.MD5Encode(vipInfo.getActivetype() + pwd + vipInfo.getTimestamp());
        if (!localSign.equals(vipInfo.getSignature())) {
            throw new BussinessException("9054", "signature校验失败");
        }
        //

    }
    public void checkVipQuery(VipInfo vipInfo) throws BussinessException {
        this.checkCpidspid(vipInfo);
        this.comonService.checkTimestamp(vipInfo.getTimestamp());
        if (StringUtils.isEmpty(vipInfo.getSignature())) {
            throw new BussinessException("9054", "signature不能为空");
        }
        CpSpBean cpSpBean = CpSpDao.getCpSp(vipInfo.getCpid(), vipInfo.getSpid());
        String pwd = cpSpBean.getPwd();
        String localSign = MD5.MD5Encode(vipInfo.getActivetype() + pwd + vipInfo.getTimestamp());
        if (!localSign.equals(vipInfo.getSignature())) {
            throw new BussinessException("9054", "signature校验失败");
        }
        this.mobService.checkVipMob(vipInfo);
    }

    /**
     * 资格校验
     *
     * @param vipInfo
     * @throws BussinessException
     */
    public void checkVip(VipInfo vipInfo,ActivityLimitConfig config) throws BussinessException {
        this.checkRedis(vipInfo);
        this.checkCpidspid(vipInfo);
        this.checkAiqy20(vipInfo, config);
        comonService.checkTimestamp(vipInfo.getTimestamp());
        comonService.checkQueryVipSign(vipInfo);
        mobService.checkVipMob(vipInfo);
    }

    public void checkCpidspid(VipInfo activityInfo)throws BussinessException{
        if(StringUtils.isEmpty(activityInfo.getCpid()) || StringUtils.isEmpty(activityInfo.getSpid())){
            throw new BussinessException("9054", "cpid/spid不能为空");
        }
    }

    /**
     * 活动抽奖
     *
     * @param activityInfo
     * @throws BussinessException
     */
    public void giveActive(VipInfo activityInfo,ActivityLimitConfig config) throws BussinessException {
        String result="";
        this.queryVip(activityInfo,config);
        String userval="";
        try {
            userval=AESUtil.decrypt(activityInfo.getUserval(),"6{Gd_J5(f8g5R[Qb");
        } catch (Exception e) {
            throw new BussinessException("9056", "userval解密错误");
        }
        if(!UsermobUtil.isValid(userval)){
            throw new BussinessException("9055", "userval不正确");
        }
        int count=activeDao.getShareOrderRelation(userval,activityInfo.getCpid(),activityInfo.getSpid());

        if(count<=0){
            throw new BussinessException("9082", "分享用户未订购该产品");
        }
        activityInfo.setUserval(userval);

        if("mangtv16".equals(activityInfo.getCpid()) && "1006".equals(activityInfo.getSpid())){
            this.joinActive(activityInfo);


        }else{
            throw new BussinessException("9088", "该cp暂未开放");
        }
        System.out.println("JoinInfo="+ JSON.toJSONString(activityInfo));
    }


    /**
     *  分享记录
     * @param activityInfo
     * @param
     * @return
     */
    public void joinActive(VipInfo activityInfo)throws BussinessException{

        //是否已经领取
        int joincount = activeDao.getShareActive(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype());
        if(joincount > 0) {
            throw new BussinessException("8089", "已参加活动");
        }
        try {
            SimpleDateFormat smf = new SimpleDateFormat("yyyyMMdd");
            String activetime = smf.format(new Date());
            if (new SimpleDateFormat("yyyyMMddHHmmss").parse(activetime + "160000").getTime() < System.currentTimeMillis()) {
                //当前时间大于当天四点  记录活动时间为明天
                Date sDate =new Date();
                Calendar c = Calendar.getInstance();
                c.setTime(sDate);
                c.add(Calendar.DAY_OF_MONTH, 1);
                sDate = c.getTime();
                System.out.println("Date结束日期+1 " +smf.format(sDate));
                activetime=smf.format(sDate);
            }
            //记录
            activeDao.insertShare(activityInfo.getUserid(), activityInfo.getUserval(), activetime,activityInfo.getCpid(),activityInfo.getSpid(),activityInfo.getActivetype());
        }catch (Exception e){
            ErrorLOGGER.error("处理异常"+e.getMessage());
            throw new BussinessException("6088", "处理异常");
        }
    }



    /**
     * 查询排行榜及当前用户排名
     * @param vipInfo
     * @throws BussinessException
     */
    public Map<String,Object> queryRank(VipInfo vipInfo,ActivityLimitConfig config) throws BussinessException {
        Map<String,Object> result=new HashMap<>();
        SimpleDateFormat smf=new SimpleDateFormat("yyyyMMdd");
        String activetime = smf.format(new Date());
        try {
            if (new SimpleDateFormat("yyyyMMddHHmmss").parse(activetime + "160000").getTime() < System.currentTimeMillis()) {
                //当前时间大于当天四点  记录活动时间为明天
                Date sDate = new Date();
                Calendar c = Calendar.getInstance();
                c.setTime(sDate);
                c.add(Calendar.DAY_OF_MONTH, 1);
                sDate = c.getTime();
                System.out.println("Date结束日期+1 " + smf.format(sDate));
                activetime = smf.format(sDate);
            }
        }catch (Exception e){
            ErrorLOGGER.error("时间解析异常"+e.getMessage());
        }
        List<ActivityMgtvShareBean> beans=activeDao.getRankNow(activetime);
        int joincount=activeDao.getJoinCount(vipInfo.getCpid(),vipInfo.getSpid(),activetime);
        String rank="";

        result.put("joincount",joincount);

        for(ActivityMgtvShareBean bean:beans){
            if(!StringUtils.isEmpty(vipInfo.getUserid())) {
                if (vipInfo.getUserid().equals(bean.getUserid())) {
                    rank = bean.getRank();
                }
            }
            bean.setUserid(bean.getUserid().substring(0,3)+"****"+bean.getUserid().substring(7,11));
        }
        result.put("ranks",beans);
        if(!StringUtils.isEmpty(rank)){
            result.put("userrank",rank);
        }

        return  result;
    }


    /**
     * 查询获奖记录
     * @throws BussinessException
     */
    public String queryRecord(VipInfo vipInfo) throws BussinessException {
        List<ActivityMgtvShareBean> beans=new ArrayList<>();
        try {
            SimpleDateFormat smf = new SimpleDateFormat("yyyyMMdd");
            String activetime = smf.format(new Date());
            //判断当前时间是否大于今日四点

            if (new SimpleDateFormat("yyyyMMddHHmmss").parse(activetime + "160000").getTime() < System.currentTimeMillis()) {

                //判断今日是否有生成获奖记录
                int size = activeDao.getTodayCount();
                if (size <= 0) {
                    insertPrize(vipInfo,activetime);
                }
            }


        }catch (Exception e){
            ErrorLOGGER.error("生成获奖名单异常"+e.getMessage());
            throw new BussinessException("8099", "获奖名单暂未公布");
        }
        beans = activeDao.getPrize();
        if(beans!=null && beans.size()>0){
            for(ActivityMgtvShareBean bean:beans){
                bean.setUserid(bean.getUserid().substring(0,3)+"****"+bean.getUserid().substring(7,11));
            }
        }else{
            throw new BussinessException("8099", "获奖名单暂未公布");
        }
        return  JSON.toJSONString(beans);
    }



    public void insertPrize(VipInfo vipInfo,String activetime){
        long count = addRedis("joinMgtvSharePrize_" + vipInfo.getActivetype() + "_" +activetime);
        if (count > 1) {
            return;
        }

        //生成获奖记录
        List<ActivityMgtvShareBean> beanList=activeDao.getTodayRank(activetime);
        System.out.println("生成今日中奖名单"+JSON.toJSONString(beanList));
        for(ActivityMgtvShareBean bean:beanList){
            activeDao.insertRank(bean.getUserid(),bean.getRank(),bean.getPrize(),bean.getActivetime(),bean.getCount());
            String content=bean.getContent().replace("#userid#",bean.getUserid().substring(0,3)+"****"+bean.getUserid().substring(7,11));
            sendSMS(bean.getUserid(),content);
        }
    }

    private static String sendSMS (String userid,String content){
        try {
            String smsid="1907120";
            String key= "wd9sjdh2";

            String timestamp = System.currentTimeMillis()/1000+"";
            String usermob =  DesUtil.encode(userid,  key);
            String sign = SHAUtil.shaEncode(smsid+usermob+timestamp+content+key).toLowerCase();

            usermob = URLEncoder.encode(usermob,"utf-8");
            content = URLEncoder.encode(content,"utf-8");
            String url = "http://111.206.133.54/smsbusi/sms/send?userid="+usermob+"&smsid="+smsid+"&timestamp="+timestamp+"&sign="+sign+"&content="+content;
            String resp = URLUtil.get(url);

            System.out.println(url+","+resp);
            System.out.println(resp);
            return resp;
        } catch (Exception e) {
            e.printStackTrace();
        }

        return null;
    }

    /**
     * 检查是否是活动且有效
     * @param vipInfo
     * @throws BussinessException
     */
    public void checkAiqy20(VipInfo vipInfo,ActivityLimitConfig config) throws BussinessException {
        String activetype=config.getActivetype();
        String activeend=config.getActiveendtime();

        if(!activetype.equals(vipInfo.getActivetype())){
            throw new BussinessException("9083", "该活动无效");
        }

        try {
            if(new SimpleDateFormat("yyyyMMddHHmmss").parse(activeend).getTime()<System.currentTimeMillis()){
                throw new BussinessException("9083", "该活动已经结束");
            }
        } catch (Exception e) {
            if (e instanceof BussinessException) {
                throw (BussinessException)e;
            }else{
                ErrorLOGGER.error("", e);
            }
        }
    }


    /**
     *
     * @param activityInfo
     * @throws BussinessException
     */
    public void queryVip(VipInfo activityInfo,ActivityLimitConfig config) throws BussinessException {
        String activetype=config.getActivetype();
        String activeend=config.getActiveendtime();
        //是否已经抽奖
        int joincount = activeDao.getShareActive(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid(), activityInfo.getActivetype());
        if(joincount > 0) {
            throw new BussinessException("6081", "已参与活动");
        }

        if(activityInfo.getUserval().equals(activityInfo.getUserid())){
            throw new BussinessException("6082", "分享人与订购人不能一致");
        }
        //是否新用户有效订购
        String orderid = activeDao.getOrderRelation(activityInfo.getUserid(), activityInfo.getCpid(), activityInfo.getSpid());
        if(orderid==null ){
            throw new BussinessException("6082", "用户不具备活动资格");
        }
        try {
            String ordertime = orderid.substring(0, 8);
            System.out.println("第一次订购时间" + ordertime);
            SimpleDateFormat smf = new SimpleDateFormat("yyyyMMdd");
            if (smf.parse(ordertime).getTime()<smf.parse("20200901").getTime() ||smf.parse(ordertime).getTime() > smf.parse(activeend).getTime()) {
                throw new BussinessException("8088", "非新用户不具备活动资格");
            }

            /*int count=activeDao.orderLogCount(activityInfo.getUserid(),activityInfo.getCpid(),activityInfo.getSpid());
            if(count>1){
                throw new BussinessException("8088", "非新用户不具备活动资格");
            }*/
        }catch (Exception e){
            if (e instanceof BussinessException) {
                throw (BussinessException)e;
            }else{
                ErrorLOGGER.error("", e);
            }
        }

    }

    public void queryPrize(VipInfo activityInfo)throws BussinessException {
        int  joincount = this.activeDao.getPrizeCount(activityInfo.getUserid());
        if (joincount == 0) {
            throw new BussinessException("6083", "您未中奖哦！");
        }
    }

    public String savePrize(UserPrizeBean activityInfo) throws BussinessException {
        int  joincount = this.activeDao.getPrizeCount(activityInfo.getUserid());
        if (joincount == 0) {
            throw new BussinessException("6083", "您未中奖哦！");
        }  else {
            String resultcode;
            try {
                String id = this.activeDao.getSavePrize(activityInfo.getUserid());
                if (id == null) {
                    throw new BussinessException("6086", "您已登记中奖信息！");
                } else {
                    resultcode=this.activeDao.savePrize( activityInfo.getUsername(), activityInfo.getAddress(), activityInfo.getPhone(),id)+"";
                    return resultcode;
                }
            } catch (Exception var6) {
                var6.printStackTrace();
                if (var6 instanceof BussinessException) {
                    resultcode = ((BussinessException)var6).getCode();
                    String errrorinfo = ((BussinessException)var6).getMessage();
                    throw new BussinessException(resultcode, errrorinfo);
                } else {
                    throw new BussinessException("6084", "保存用户领奖信息异常");
                }
            }
        }
    }



    public void checkRedis(VipInfo vipInfo) throws BussinessException {
        try {
            long count = redisService.incr("joinMgtvShare_" + vipInfo.getActivetype() + "_" + vipInfo.getUserid());
            if (count > 1) {
                throw new BussinessException("8001", "并发请求，请稍后再试");
            }
        } catch (Exception e) {
            if (e instanceof BussinessException) {
                throw e;
            }else{
                ErrorLOGGER.error("", e);
            }
        }
    }



    /**
     * ip校验
     */
    public void checkIp(ActivityLimitConfig config)throws BussinessException {

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        if(org.apache.commons.lang.StringUtils.isNotEmpty(config.getLimitip())) {
            System.out.println("ip============>"+config.getLimitip());
            List<String> list = Arrays.asList(config.getLimitip().split(","));
            System.out.println("listIP:"+list);
            String ip = IpAddrUtil.getIpAddr(request);
            System.out.println("ip:"+ip);
            if(!list.contains(ip)) {
                throw new BussinessException("10009", "ip地址校验失败"+ip);
            }
        }
    }

}
