package com.chinacreator.video.service;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.xml.bind.DatatypeConverter;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.DateFormatUtils;
import org.n3r.ecaop.client.EcAopClient;
import org.n3r.ecaop.client.EcAopMethod;
import org.n3r.ecaop.client.EcAopResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.video.bean.AsynOrderBean;
import com.chinacreator.video.bean.CpSpBean;
import com.chinacreator.video.bean.OrderInfo;
import com.chinacreator.video.bean.VipInfo;
import com.chinacreator.video.bean.WhiteBean;
import com.chinacreator.video.dto.QueryAsynOutPipe;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.mapper.AreaDao;
import com.chinacreator.video.mapper.AsynDao;
import com.chinacreator.video.mapper.CpSpDao;
import com.chinacreator.video.mapper.OrderDao;
import com.chinacreator.video.mapper.SysDao;
import com.chinacreator.video.mapper.WhiteDao;
import com.chinacreator.video.util.URLUtil;
import com.chinacreator.video.util.VacUtil;

@Component
public class AsynOrderService {

	@Autowired
	private ComonService comonService;

	@Autowired
	private ChannelValidate channelValidate;

	@Autowired
	private CpSpService cpSpService;

	@Autowired
	private MobService mobService;

	@Autowired
	private AsynDao asynDao;

	@Autowired
	private OrderidValidate orderidValidate;

	@Autowired
	private CpSpDao cpSpDao;

	@Autowired
	private AreaDao areaDao;

	@Autowired
	private GiveVipService giveVipService;

	@Autowired
	private WhiteDao whiteDao;

	@Autowired
	private VacControlService vacControlService;

	@Autowired
	private EcbService ecbService;
	
	@Autowired
	private SysDao sysDao;
	
	@Autowired
	private OrderDao orderDao;

	@Autowired
	private UnifiedAuthService unifiedAuthService;
	
	private static Logger ErrorLOGGER = LoggerFactory.getLogger("ERROR"); 

	public void check(AsynOrderBean asynOrderBean) throws BussinessException {
		comonService.checkTimestamp(asynOrderBean.getTimestamp());
		channelValidate.validate(asynOrderBean.getChannel());
		comonService.checkSignAnsy(asynOrderBean);
		String data = asynOrderBean.getData();
		if (StringUtils.isEmpty(data)) {
			throw new BussinessException("9073", "data不能为空");
		}
		JSONObject obj = JSON.parseObject(data);
		asynOrderBean.setCpid(obj.getString("cpid"));
		asynOrderBean.setSpid(obj.getString("spid"));
		cpSpService.checkSp(asynOrderBean.getCpid(), asynOrderBean.getSpid());
		asynOrderBean.setUserid(obj.getString("userid"));
		mobService.checkAsynMob(asynOrderBean);
		orderidValidate.validateRecive(asynOrderBean.getOrderid(), asynOrderBean.getChannel());
		asynOrderBean.setUserval(obj.getString("userval"));
		asynOrderBean.setUsertype(obj.getString("usertype"));
	}

	public void checkQuery(AsynOrderBean asynOrderBean) throws BussinessException {
		comonService.checkTimestamp(asynOrderBean.getTimestamp());
		comonService.checkSignAnsyQuery(asynOrderBean);
		orderidValidate.validateQuery(asynOrderBean.getOrderid(), asynOrderBean.getChannel());
	}

	public void recive(AsynOrderBean asynOrderBean) {
		asynOrderBean.setOrderstatus("1");
		asynOrderBean.setVipstatus("1");
		asynOrderBean.setSyncstatus("1");
		asynDao.insert(asynOrderBean);
	}

	public QueryAsynOutPipe query(AsynOrderBean asynOrderBean) throws BussinessException {
		QueryAsynOutPipe outpipe = new QueryAsynOutPipe();
		asynOrderBean = asynDao.query(asynOrderBean.getOrderid(), asynOrderBean.getChannel());
		if (asynOrderBean == null) {
			throw new BussinessException("9061", "无该订单号信息");
		}
		if (asynOrderBean.getSyncstatus().endsWith("1")) {
			throw new BussinessException("9060", "该订单暂未处理完成");
		}
		outpipe.setUserid(asynOrderBean.getUserid());
		outpipe.setAction(asynOrderBean.getAction());
		outpipe.setActiontime(asynOrderBean.getSynctime());
		outpipe.setVipstatus(asynOrderBean.getVipstatus());
		outpipe.setOrderstatus(asynOrderBean.getOrderstatus());
		return outpipe;
	}

	public void notifyOrder(String orderid) throws Exception {
		AsynOrderBean asynOrderBean = new AsynOrderBean();
		asynOrderBean = asynDao.queryUnNotify(orderid);
		if(asynOrderBean==null) {
			throw new BussinessException("1040", "无该订单");
		}
		if(!asynOrderBean.getSyncstatus().equals("1")) {
			throw new BussinessException("1040", "该订单已处理完成");
		}
		String resultcode="0";
		String errorinfo = "";
		try {
			CpSpBean cpSpbean = cpSpDao.getCpSp(asynOrderBean.getCpid(), asynOrderBean.getSpid());
			OrderInfo orderInfo = new OrderInfo();
			orderInfo.setUserid(asynOrderBean.getUserid());
			orderInfo.setCpid(asynOrderBean.getCpid());
			orderInfo.setSpid(asynOrderBean.getSpid());
			orderInfo.setSyncstatus("0");
			orderInfo.setMutex(cpSpbean.getMutex());
			orderInfo.setChannel(asynOrderBean.getChannel());
			orderInfo.setOrderchannel(asynOrderBean.getChannel());
			orderInfo.setTimes(System.currentTimeMillis() + "");
			Map<String, String> map = areaDao.getArea(orderInfo.getUserid());
			if (map != null) {
				orderInfo.setProvince(map.get("province") == null ? "" : map.get("province"));
				orderInfo.setArea(map.get("area") == null ? "" : map.get("area"));
			}
//			if (asynOrderBean.getAction().equals("1")) {
//				orderService.order(orderInfo);
//			} else {
//				WhiteBean whiteBean = whiteDao.getUser(orderInfo.getUserid(), orderInfo.getCpid());
//				if (whiteBean == null || whiteBean.getVac().equals("1")) {
//					ecbService.queryEcb(orderInfo.getUserid(), orderInfo.getCpid(), orderInfo.getSpid(),
//							orderInfo.getProvince());
//					vacControlService.canOrder(orderInfo);
//					String result = VacUtil.vac(orderInfo.getUserid(), cpSpbean.getSpcode(), cpSpbean.getVacproductid(),
//							cpSpbean.getType());
//					vacControlService.saveVacOrder(orderInfo, result);
//					if (!result.equals("0")) {
//						throw new BussinessException(result, "订购失败，VAC错误");
//					}
//				}
//			}
			order(orderInfo, cpSpbean, asynOrderBean.getAction());
		} catch (Exception e) {
			if (e instanceof BussinessException) {
				resultcode = ((BussinessException) e).getCode();
				errorinfo = ((BussinessException) e).getMessage();
			} else {
				resultcode = "8000";
				errorinfo = "系统忙，请稍后再试";
			}
			throw e;
		} finally {
			asynDao.updateOrderStatus(orderid,resultcode,errorinfo);
		}
	}

	public void order(OrderInfo orderInfo,CpSpBean cpSpbean,String action) throws Exception {
		WhiteBean whiteBean = whiteDao.getUser(orderInfo.getUserid(), orderInfo.getCpid());
		if (whiteBean != null) {
			if (whiteBean.getType().equals("1")) {
				throw new BussinessException("9065", "黑名单用户禁止订购");
			}
		}
		if (whiteBean == null || whiteBean.getVac().equals("1")) {
			ecbService.queryEcb(orderInfo.getUserid(), orderInfo.getCpid(), orderInfo.getSpid(),
					orderInfo.getProvince());
			vacControlService.canOrder(orderInfo);
			//调用续订同步接口  同步订单
			unifiedAuthService.remainUserSycn(orderInfo);

			String result = VacUtil.vac(orderInfo.getUserid(), cpSpbean.getSpcode(), cpSpbean.getVacproductid(),
					cpSpbean.getType(), orderInfo.getId() == null ? "" : orderInfo.getId());
			vacControlService.saveVacOrder(orderInfo, result);
			if (!result.equals("0")) {
				throw new BussinessException(result, "订购失败，VAC错误");
			}
		}
		orderInfo.setOrdertime(sysDao.getSysTime());
		orderInfo.setEffecttime(sysDao.getEfftTime());
		orderInfo.setOrderchannel(orderInfo.getChannel());
		orderInfo.setEndtime(sysDao.getLastDay());
		orderDao.order(orderInfo);
	}
	
	
	public void notifyVip(String orderid) throws Exception {
		AsynOrderBean asynOrderBean = new AsynOrderBean();
		asynOrderBean = asynDao.queryUnNotify(orderid);
		if(asynOrderBean==null) {
			throw new BussinessException("1040", "无该订单");
		}
		String resultcode="0";
		String errorinfo = "";
		try {
			if(!asynOrderBean.getVipstatus().equals("1")) {
				throw new BussinessException("1040", "该订单已处理完成");
			}
			if(!asynOrderBean.getResultcode().equals("0")) {
				throw new BussinessException(asynOrderBean.getResultcode(), asynOrderBean.getErrorinfo());
			}
			VipInfo activityInfo = new VipInfo();
			activityInfo.setUserid(asynOrderBean.getUserid());
			activityInfo.setCpid(asynOrderBean.getCpid());
			activityInfo.setSpid(asynOrderBean.getSpid());
			activityInfo.setUsertype(asynOrderBean.getUsertype());
			activityInfo.setUserval(asynOrderBean.getUserval());
			activityInfo.setActivetype("14");
			activityInfo.setVipname("");
			activityInfo.setChannel(asynOrderBean.getChannel());
			if (!asynOrderBean.getAction().equals("1")) {
				activityInfo.setType(1);
			}
			giveVipService.give(activityInfo);
		} catch (Exception e) {
			if (e instanceof BussinessException) {
				resultcode = ((BussinessException) e).getCode();
				errorinfo = ((BussinessException) e).getMessage();
			} else {
				resultcode = "8000";
				errorinfo = "系统忙，请稍后再试";
			}
			throw e;
		} finally {
			asynDao.updateVipStatus(orderid,resultcode,errorinfo);
		}
	}

	public void notifyResult(String orderid) throws Exception {
		AsynOrderBean asynOrderBean = new AsynOrderBean();
		asynOrderBean = asynDao.queryUnNotify(orderid);
		if(asynOrderBean==null) {
			throw new BussinessException("1040", "无该订单");
		}
		if(!asynOrderBean.getSyncstatus().equals("1")) {
			throw new BussinessException("1040", "该订单已处理完成");
		}
		if(asynOrderBean.getOrderstatus().equals("1")) {
			throw new BussinessException("1042", "该订单订购还未处理");
		}
		if(asynOrderBean.getVipstatus().equals("1")) {
			throw new BussinessException("1043", "该订单会员还未赠送");
		}
		try {
			getResult(asynOrderBean);
		} catch (Exception e) {
			ErrorLOGGER.error("电商通知接口错误", e);
			asynOrderBean.setRespcode("-1");
			throw e;
		} finally {
			this.asynDao.updateByOrderId(asynOrderBean);
		}
	}

	public void getResult(AsynOrderBean asynOrderBean) throws Exception {
		String result = anycAop(asynOrderBean);
		asynOrderBean.setRetrunput(result);
		JSONObject obj = JSON.parseObject(result);
		asynOrderBean.setRespcode((String) obj.get("respcode"));
		if (obj.get("respcode").equals("0000")) {
			asynOrderBean.setSyncstatus("0");
		}
	}

	private String anycAop(AsynOrderBean asynOrderBean) {
		String asycurl = "http://open.10010.com/openservlet";
		String appkey = "com.aop.app.unicomonline";
		String method = "com.ssp.method.tencentvideovipinform";
		String apptx = System.currentTimeMillis() + "";
		String securet = "fKldgAqrjWAMW67FLGIMja9OcUYF50TXToFrx3jF9Ka+eG08cjySLkC+Ns7KnyTfKCAa+G35uvDqODqzaAtFyA==";
		String timestamp = DateFormatUtils.format(new Date(), "yyyy-MM-dd HH:mm:ss");
		String outorder = asynOrderBean.getOrderid();
		String usernumber = asynOrderBean.getUserid();
		String usertype = asynOrderBean.getUsertype();
		String userval = asynOrderBean.getUserval();
		String busitime = DateFormatUtils.format(new Date(), "yyyyMMddHHmmss");
		String resultcode = asynOrderBean.getResultcode();

		EcAopClient aopClient = new EcAopClient(asycurl,appkey,securet);
		EcAopMethod ecAopMethod = aopClient.createEcAopMethod(method, Map.class);
		Map<String, String> map = new HashMap<>();
		map.put("appkey", appkey);
		map.put("method", method);
		map.put("apptx", apptx);
		map.put("timestamp", timestamp);
		map.put("outorder", outorder);
		map.put("usernumber", usernumber);
		map.put("usertype", usertype);
		map.put("userval", userval);
		map.put("busitime", busitime);
		map.put("resultcode", resultcode);

		EcAopResult aopResult = ecAopMethod.exec(map);

		return aopResult.getResponse();
	}

//	public String anyc(AsynOrderBean asynOrderBean) throws Exception {
//		String asycurl = "http://open.10010.com/openservlet";
//		String appkey = "com.aop.app.unicomonline";
//		String method = "com.ssp.method.tencentvideovipinform";
//		String apptx = System.currentTimeMillis() + "";
//		String securet = "fKldgAqrjWAMW67FLGIMja9OcUYF50TXToFrx3jF9Ka+eG08cjySLkC+Ns7KnyTfKCAa+G35uvDqODqzaAtFyA==";
//		String timestamp = DateFormatUtils.format(new Date(), "yyyy-MM-dd HH:mm:ss");
//		String outorder = asynOrderBean.getOrderid();
//		String usernumber = asynOrderBean.getUserid();
//		String usertype = asynOrderBean.getUsertype();
//		String userval = asynOrderBean.getUserval();
//		String busitime = DateFormatUtils.format(new Date(), "yyyyMMddHHmmss");
//		String resultcode = asynOrderBean.getResultcode();
//		//String resultinfo = asynOrderBean.getErrorinfo();
//		List<String> signList = new ArrayList<>();
//		Map<String, String> map = new HashMap<>();
//		signList.add("appkey");
//		map.put("appkey", appkey);
//		signList.add("method");
//		map.put("method", method);
//		signList.add("apptx");
//		map.put("apptx", apptx);
//		signList.add("timestamp");
//		map.put("timestamp", timestamp);
//		signList.add("outorder");
//		map.put("outorder", outorder);
//		signList.add("usernumber");
//		map.put("usernumber", usernumber);
//		signList.add("usertype");
//		map.put("usertype", usertype);
//		signList.add("userval");
//		map.put("userval", userval);
//		signList.add("busitime");
//		map.put("busitime", busitime);
//		signList.add("resultcode");
//		map.put("resultcode", resultcode);
////		if (StringUtils.isNotEmpty(resultinfo)) {
////			signList.add("resultinfo");
////			map.put("resultinfo", resultinfo);
////		}
//		String sign = getSign(signList, securet, map);
//		String aurl = asycurl + "?outorder=" + outorder + "&usernumber=" + usernumber + "&usertype=" + usertype
//				+ "&userval=" + userval + "&busitime=" + busitime + "&resultcode=" + resultcode + "&appkey=" + appkey
//				+ "&sign=" + URLEncoder.encode(sign, "utf-8") + "&apptx=" + apptx + "&timestamp=" + timestamp
//				+ "&method=" + method;
//		System.out.println(aurl);
//		String result = URLUtil.postForm(aurl, "", 5000);
//		return result;
//	}
//
//	public String getSign(List<String> list, String securet, Map<String, String> map)
//			throws UnsupportedEncodingException, NoSuchAlgorithmException {
//		StringBuilder strb = new StringBuilder();
//		strb.append(securet);
//		Collections.sort(list);
//		for (String string : list) {
//			if (StringUtils.isNotEmpty(map.get(string))) {
//				strb.append("$" + string + "$" + map.get(string));
//			}
//		}
//		strb.append("$" + securet);
//		System.out.println("sign加密前======》" + strb.toString());
//		MessageDigest md = MessageDigest.getInstance("MD5");
//		md.update(strb.toString().getBytes("UTF-8"));
//		byte[] digest = md.digest();
//		String s = DatatypeConverter.printBase64Binary(digest);
//		System.out.println("md5加密后======》" + s);
//		String sign = s;
//		System.out.println("base64加密后======》" + sign);
//		return sign;
//	}
}
