package com.chinacreator.video.service;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import com.chinacreator.video.bean.VipInfo;
import com.alibaba.fastjson.JSON;
import com.chinacreator.video.bean.ActivityMsgBean;
import com.chinacreator.video.bean.AsynOrderBean;
import com.chinacreator.video.bean.CpSpBean;
import com.chinacreator.video.bean.OrderInfo;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.mapper.ChannelDao;
import com.chinacreator.video.mapper.CommConfDao;
import com.chinacreator.video.mapper.CpSpDao;
import com.chinacreator.video.mapper.DictionaryDao;
import com.chinacreator.video.util.MD5;

@Component
public class ComonService {
	
	private static Logger log = Logger.getLogger(ComonService.class);

	@Autowired
	private CpSpDao CpSpDao;

	@Autowired
	private ChannelDao channelDao;
	
	@Autowired
	private DictionaryDao dictionaryDao;

	@Autowired
	private CommConfDao commConfDao;

	public void checkOrder(OrderInfo orderInfo) throws BussinessException {
		checkTimestamp(orderInfo.getTimestamp());
		checkSign(orderInfo);
	}

	public void checkSignAnsy(AsynOrderBean asynOrderBean) throws BussinessException {
		String sign = asynOrderBean.getSignature();
		if (StringUtils.isEmpty(sign)) {
			throw new BussinessException("9054", "signature不能为空");
		}
		String pwd = channelDao.queryChannel(asynOrderBean.getChannel()).getPassword();
		String localSign = MD5.MD5Encode(asynOrderBean.getOrderid() + asynOrderBean.getChannel()
				+ asynOrderBean.getTimestamp() + asynOrderBean.getAction() + pwd);
		if (!localSign.equals(asynOrderBean.getSignature())) {
			throw new BussinessException("9054", "signature校验失败");
		}
	}

	public void checkSignAnsyQuery(AsynOrderBean asynOrderBean) throws BussinessException {
		System.out.println(JSON.toJSONString(asynOrderBean));
		String sign = asynOrderBean.getSignature();
		if (StringUtils.isEmpty(sign)) {
			throw new BussinessException("9054", "signature不能为空");
		}
		String pwd = channelDao.queryChannel(asynOrderBean.getChannel()).getPassword();
		String localSign = MD5.MD5Encode(
				asynOrderBean.getOrderid() + asynOrderBean.getChannel() + asynOrderBean.getTimestamp() + pwd);
		if (!localSign.equals(asynOrderBean.getSignature())) {
			throw new BussinessException("9054", "signature校验失败");
		}
	}

	public void checkSign(OrderInfo orderInfo) throws BussinessException {
		String sign = orderInfo.getSignature();
		if (StringUtils.isEmpty(sign)) {
			throw new BussinessException("9054", "signature不能为空");
		}
		CpSpBean cpSpBean = CpSpDao.getCpSp(orderInfo.getCpid(), orderInfo.getSpid());
		String pwd = cpSpBean.getPwd();
		String localSign = MD5.MD5Encode(
				orderInfo.getCpid() + orderInfo.getSpid() + orderInfo.getUserid() + pwd + orderInfo.getTimestamp());
		if (!localSign.equals(orderInfo.getSignature())) {
			throw new BussinessException("9054", "signature校验失败");
		}
	}

	public void checkSignChannel(OrderInfo orderInfo) throws BussinessException {
		String sign = orderInfo.getSignature();
		if (StringUtils.isEmpty(sign)) {
			throw new BussinessException("9054", "signature不能为空");
		}
		String pwd = channelDao.queryChannel(orderInfo.getChannel()).getPassword();
		String localSign = MD5.MD5Encode(orderInfo.getOrderid() + orderInfo.getType() + orderInfo.getCpid()
				+ orderInfo.getSpid() + orderInfo.getUserid() + pwd + orderInfo.getTimestamp());
		if (!localSign.equals(orderInfo.getSignature())) {
			throw new BussinessException("9054", "signature校验失败");
		}
	}

	public void checkSignChannelQuery(OrderInfo orderInfo) throws BussinessException {
		String sign = orderInfo.getSignature();
		if (StringUtils.isEmpty(sign)) {
			throw new BussinessException("9054", "signature不能为空");
		}
		String pwd = channelDao.queryChannel(orderInfo.getChannel()).getPassword();
		String localSign = MD5.MD5Encode(
				orderInfo.getCpid() + orderInfo.getSpid() + orderInfo.getUserid() + pwd + orderInfo.getTimestamp());
		if (!localSign.equals(orderInfo.getSignature())) {
			throw new BussinessException("9054", "signature校验失败");
		}
	}

	public void checkGetVipSign(VipInfo activityInfo) throws BussinessException {
		String sign = activityInfo.getSignature();
		if (StringUtils.isEmpty(sign)) {
			throw new BussinessException("9054", "signature不能为空");
		}
		CpSpBean cpSpBean = CpSpDao.getCpSp(activityInfo.getCpid(), activityInfo.getSpid());
		String pwd = cpSpBean.getPwd();
		String localSign = MD5
				.MD5Encode(activityInfo.getUserid() + activityInfo.getUserval() + pwd + activityInfo.getTimestamp());
		if (!localSign.equals(activityInfo.getSignature())) {
			throw new BussinessException("9054", "signature校验失败");
		}
	}

	public void checkGetMsgSign(ActivityMsgBean activityMsgBean) throws BussinessException {
		String sign = activityMsgBean.getSignature();
		if (StringUtils.isEmpty(sign)) {
			throw new BussinessException("9054", "signature不能为空");
		}
		CpSpBean cpSpBean = CpSpDao.getCpSp(activityMsgBean.getCpid(), activityMsgBean.getSpid());
		Map<String, String> map = new TreeMap<>();
		map.put("cpid", activityMsgBean.getCpid());
		map.put("spid", activityMsgBean.getSpid());
		map.put("userid", activityMsgBean.getUserid());
		map.put("serialid", activityMsgBean.getSerialid());
		map.put("timestamp", activityMsgBean.getTimestamp());
		map.put("type", activityMsgBean.getType());
		StringBuilder str = new StringBuilder();
		for (String key : map.keySet()) {
			str.append(key);
			str.append(map.get(key));
		}
		String pwd = cpSpBean.getPwd();
		str.append(pwd);
		System.out.println("str="+str.toString());
		String localSign = MD5.MD5Encode(str.toString());
		System.out.println("localSign="+localSign);
		System.out.println("Signature="+activityMsgBean.getSignature());
		if (!localSign.equals(activityMsgBean.getSignature())) {
			throw new BussinessException("9054", "signature校验失败");
		}
	}

	public void checkQueryVipSign(VipInfo activityInfo) throws BussinessException {
		String sign = activityInfo.getSignature();
		if (StringUtils.isEmpty(sign)) {
			throw new BussinessException("9054", "signature不能为空");
		}
		CpSpBean cpSpBean = CpSpDao.getCpSp(activityInfo.getCpid(), activityInfo.getSpid());
		String pwd = cpSpBean.getPwd();
		String localSign = MD5.MD5Encode(activityInfo.getUserid() + pwd + activityInfo.getTimestamp());
		if (!localSign.equals(activityInfo.getSignature())) {
			throw new BussinessException("9054", "signature校验失败");
		}
	}

	public void checkChannelQueryVipSign(VipInfo activityInfo) throws BussinessException {
		String sign = activityInfo.getSignature();
		if (StringUtils.isEmpty(sign)) {
			throw new BussinessException("9054", "signature不能为空");
		}
		String pwd = channelDao.queryChannel(activityInfo.getChannel()).getPassword();
		String localSign = MD5.MD5Encode(activityInfo.getUserid() + activityInfo.getActivetype()
				+ activityInfo.getChannel() + pwd + activityInfo.getTimestamp());
		if (!localSign.equals(activityInfo.getSignature())) {
			throw new BussinessException("9054", "signature校验失败");
		}
	}

	public void checkTimestamp(String strtimestamp) throws BussinessException {
		if (StringUtils.isEmpty(strtimestamp)) {
			throw new BussinessException("9052", "timestamp参数不能为空");
		}
		long timestamp = Long.parseLong(strtimestamp);
		long nowTimestamp = System.currentTimeMillis() / 1000;
		if ((nowTimestamp - timestamp) > 300) {
			throw new BussinessException("9053", "timestamp已失效");
		}
	}

	/**
	 * 验证省份办理限制
	 * @param orderInfo
	 * @throws Exception
	 */
	public void  valiProvinceLimit(OrderInfo orderInfo) throws BussinessException {
		//验证是否通过，true是，false否
		boolean haspass = true;
		String errorInfo = "";
		String errorcode = "";
		List<HashMap> confList = null;
		try {
			//不限制办理的CPID
			String notProvinceLimitcpid = dictionaryDao.getValue("notProvinceLimitcpid");
			if(!StringUtils.isEmpty(notProvinceLimitcpid) && !StringUtils.isEmpty(orderInfo.getCpid())){ 
				List<String> cpids = Arrays.asList(notProvinceLimitcpid.split(","));
				if(cpids != null && cpids.contains(orderInfo.getCpid())){
					return;
				}
			}
			String province = orderInfo.getProvince();
			String area = orderInfo.getArea();
			String invokeface = orderInfo.getInvokeface();
			String type = "0"; //0订购，1退订
			if("cancel".equals(invokeface)){
				type = "1";
			}
			String channel = orderInfo.getChannel();

			if(StringUtils.isEmpty(province)){
				province = "*";
			}
			//接口标识为空，不验证
			if(StringUtils.isEmpty(invokeface)){ 
				return; 
			}
			if(StringUtils.isEmpty(province)){ 
				province = "*";
			}
			if(StringUtils.isEmpty(type)){
				type = "*";
			}
			if(StringUtils.isEmpty(channel)){
				channel = "*";
			}
			//查询是否有省份限制
			confList = commConfDao.qryProvInvokeLimit(province, channel, invokeface, type);
			if(confList != null && confList.size() > 0){
				haspass = false;
				errorcode = "9003";
				errorInfo = "您号码的归属地暂未开放该业务，敬请期待。";
				if(!"*".equals(province)){
					errorInfo = "您号码的归属地"+province+"暂未开放该业务，敬请期待。";
				}
				if("广东".equals(province) && "0".equals(type)){
					errorInfo = "业务平台满载，正在升级中，暂时无法受理新用户订购，敬请谅解！";
				}

				throw new BussinessException(errorcode, errorInfo);
			}
		} catch (Exception e) {
			if(e instanceof BussinessException) {
				throw e;
			}else{
				e.printStackTrace();
				if(orderInfo != null){
					log.error(orderInfo.getUserid()+"，"+orderInfo.getSpid()+"，order"+"，验证办理省份限制出现异常，"+e.getMessage(),e);
				}
			}
		} finally{
			if(!haspass){
				log.info(haspass+"，confList=>"+confList+"，province=>"+orderInfo.getProvince()+"，userid=>"+orderInfo.getUserid()+"，invokeface=>order"+"，channel=>"+orderInfo.getChannel()+"，type=>"+orderInfo.getType()+"，"+"办理省份限制返回信息：errorcode=>"+errorcode+"，errorInfo=>"+errorInfo);
			}
		}
	}
	
	public static void main(String[] args) throws Exception {
		System.out.println(MD5.MD5Encode("jTjSV5V08j0XHeLZcUR13QhW9MHpjhzc/+6Tpcu0zq828Ey/hAWi3A==tenc12341520783541"));
	}

}
