package com.chinacreator.video.service;

import com.chinacreator.video.bean.*;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.log.VipLog;
import com.chinacreator.video.mapper.ActivityConfigDao;
import com.chinacreator.video.mapper.ComplementSendDao;
import com.chinacreator.video.mapper.VipGiveDao;
import com.chinacreator.video.redis.RedisService;
import com.chinacreator.video.util.IpAddrUtil;
import com.chinacreator.video.util.StrUtil;
import com.chinacreator.video.util.VipChangeUtil;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * 根据账期补送
 *
 * @Author shuiying.ou
 * @Date 2022-12-12
 */
@Service
public class ComplementSendService {

    private static final Logger log = LoggerFactory.getLogger("complementsend");


    @Autowired
    private ComplementSendDao complementSendDao;

    @Autowired
    RedisService redisService;

    @Autowired
    ActivityConfigDao activityConfigDao;

    @Autowired
    ActivityVipConfigService vipConfigService;

    @Autowired
    private VipChangeUtil vipChangeUtil;

    @Autowired
    private VipGiveDao vipGiveDao;


    public void checkSendVip(VipAccountBean vipAccount) throws BussinessException {
        String ips = complementSendDao.getLimitIp("complementSend");
        checkip(ips);
        checkredis(vipAccount);
        checkVipGiveCount(vipAccount);
    }

    /**
     * 并发控制
     *
     * @param vipAccountBean
     * @throws BussinessException
     */
    private void checkredis(VipAccountBean vipAccountBean) throws BussinessException {
        long count = redisService.incr("sendvip_" + vipAccountBean.getActivitytype() + "_" + vipAccountBean.getCpid() + "_" + vipAccountBean.getSpid() + "_" + vipAccountBean.getUserid());
        if (count > 1) {
            throw new BussinessException("8001", "并发请求，请稍后再试");
        }
    }

    public void checkVipGiveCount(VipAccountBean vipAccountBean) {
        //查询可以领取会员的个数
        try {
            if (!StringUtils.isEmpty(vipAccountBean.getFhspid())) {
                // 复合产品能领几个会员根据配置表
                ActivityConfigBean config = activityConfigDao.getConfig(vipAccountBean.getActivitytype());
                vipAccountBean.setGivecount(Integer.parseInt(config.getGivecount()));
            } else {
                // 单产品只能另一个会员
                vipAccountBean.setGivecount(1);
            }
        } catch (Exception e) {
            log.info("获取会员领取个数失败, " + vipAccountBean.getUserid());
            throw e;
        }
    }

    public VipAccountBean checkData(VipAccountBean vipAccount) throws BussinessException {
        vipAccount = complementSendDao.findById(vipAccount.getId());
        if (StringUtils.isEmpty(vipAccount.getUserid())) {
            throw new BussinessException("8010", "id数据异常");
        }
        return vipAccount;
    }

    /**
     * 根据接口名设置具体的ip限制
     *
     * @param ips
     * @throws BussinessException
     */
    private void checkip(String ips) throws BussinessException {

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        List<String> list = Arrays.asList(ips.split("\\|"));
        System.out.println("ips: " + list);
        String ip = IpAddrUtil.getIp(request).trim();
        System.out.println("返回的ip是："+ip);
        log.info("ip:" + ip);
        if (!list.contains(ip)) {
            throw new BussinessException("10009", "ip地址校验失败: " + ip);
        }
    }

    public void sendVip(VipAccountBean vipAccount) throws Exception {
        if ("3".equals(vipAccount.getVipstatus())) {
            throw new BussinessException("9999", "会员补送中，请稍后查看");
        } else if ("4".equals(vipAccount.getVipstatus())) {
            throw new BussinessException("9090", "该活动已暂停赠送会员");
        }
        // 根据userid、activitytype、month、userid=userval 查tl_vip_log表 判断当月是否有领取或者续送记录
        List<VipLogBean> vipLogs = complementSendDao.findVipLog(vipAccount);
        // 查 td_vipgive_control 该spid当前月份是否有记录，因为为复合产品时，tl_vip_log没有办法确认是哪个子产品充值的会员
        List<String> times = complementSendDao.findBySpid(vipAccount);
        // 区分单产品与复合产品
        if (!StringUtils.isEmpty(vipAccount.getFhspid())) {
            // 复合产品（包含组合产品与多选一）
            // 在当月有会员充值记录     有可能多个会员只领过一个    在达到可领取数后其余置为失效
            Integer count = null;
            // 判断当前账期中已领取或续送的会员数 与 可赠送的会员数
            count = complementSendDao.findCount(vipAccount);
            if (count >= vipAccount.getGivecount()) {
                complementSendDao.updateStatus(vipAccount);
                throw new BussinessException("8005", "会员领取数量已达到可领会员数");
            }
            if (vipLogs.size() > 0 && times.size() > 0) {
                String num = vipGiveDao.getVipGiveInfoByTime(vipAccount.getUserid(), vipAccount.getCpid(), vipAccount.getSpid(), vipAccount.getActivitytype(), vipAccount.getMonthtime());
                if (Integer.parseInt(num) > 0) {
                    // 当月该spid有会员充值记录
                    vipAccount.setViptime(times.get(0));
                    vipAccount.setVipstatus("0");
                    vipAccount.setViptype("2");   // 默认手动领取
                    if (vipLogs.size() == 1) {
                        VipLogBean vipLogBean = vipLogs.get(0);
                        if (vipLogBean.getType() != null) {
                            // 1 续送   2 领取
                            vipAccount.setViptype(1 == vipLogBean.getType() ? "1" : "2");
                        }
                    }
                    complementSendDao.updateAccount(vipAccount);
                    // 账期表同一个activitytype同一个月已经补送的数量 与 赠送中的数量 >= 可赠送的会员数时，其余的同月账期 置为失效
                    count = complementSendDao.findCount(vipAccount);
                    if (count >= vipAccount.getGivecount()) {
                        complementSendDao.updateStatus(vipAccount);
                    }
                    throw new BussinessException("8005", "会员已领取，请重新查询");

                } else {
                    // 选择补送的会员无充值记录
                    this.complementSend(vipAccount);
                    // 账期表同一个activitytype同一个月已经补送的数量 与 赠送中的数量 >= 可赠送的会员数时，其余的同月账期 置为失效
                    count = complementSendDao.findCount(vipAccount);
                    if (count >= vipAccount.getGivecount()) {
                        complementSendDao.updateStatus(vipAccount);
                    }
                }
            } else {
                // 未领 补送业务
                this.complementSend(vipAccount);
                // 账期表同一个activitytype同一个月已经补送的数量 与 赠送中的数量 >= 可赠送的会员数时，其余的同月账期 置为失效
                count = complementSendDao.findCount(vipAccount);
                if (count >= vipAccount.getGivecount()) {
                    complementSendDao.updateStatus(vipAccount);
                }
            }


        } else {

            // 单产品
            // 场景：12月补送了会员，1月补送12的会员时，会显示12月已充值过该会员，所以需要与vipgive表一起判断
            if (vipLogs.size() > 0 && vipLogs.get(0) != null && times.size() > 0 && times.get(0) != null) {
                VipLogBean vipLogBean = vipLogs.get(0);
                String givetime = times.get(0);
                // 有  更新td_vip_account记录      返回 当月已领取/续送
                if (vipLogBean.getType() != null) {
                    // 1 续送   2 领取
                    vipAccount.setViptype(1 == vipLogBean.getType() ? "1" : "2");
                }
                vipAccount.setRequestid(StringUtils.isEmpty(vipLogBean.getRequestid()) ? vipLogBean.getOrderid() : vipLogBean.getRequestid());
                vipAccount.setViptime(givetime);
                vipAccount.setVipstatus("0");
                complementSendDao.updateAccount(vipAccount);
                throw new BussinessException("8005", "会员已领取，请重新查询");
            } else {
                // 无 补送
                complementSend(vipAccount);
            }

        }


    }


    /**
     * 公共补送接口
     *
     * @param vipAccount
     * @throws Exception
     */
    private void complementSend(VipAccountBean vipAccount) throws Exception {
        try {
            // 获取补送vipid
           /* String vipid;
            String revipid;
            if ("954".equals(vipAccount.getSpid())) {
                revipid = "20730";
            } else if ("99133542".equals(vipAccount.getSpid())) {
                revipid = "20738";
            } else {
                // 用领取的vipid指向的revipid
                vipid = complementSendDao.getvipid(vipAccount.getSpid(), vipAccount.getActivitytype());
                if (!StringUtils.isEmpty(vipid)) {
                    revipid = complementSendDao.getRevipid(vipid);
                } else {
                    throw new BussinessException("9109", "补送会员配置异常，请联系管理员");
                }
            }
*/
            String revipid = getRevipid(vipAccount);
            vipAccount.setRevipid(revipid);
            // 新的补送vipid，直接用pwd
            String vippwd = complementSendDao.getVipConfig(vipAccount.getRevipid());
            if (!StringUtils.isEmpty(vippwd)) {
                vipAccount.setVippwd(vippwd);
                // 检验当前能否赠送会员
                String countnum = vipGiveDao.getVipGiveInfoByTime(vipAccount.getUserid(), vipAccount.getCpid(), vipAccount.getSpid(), vipAccount.getActivitytype(), vipAccount.getMonthtime());
                if (!countnum.equals("0")) {
                    throw new BussinessException("8004", "该月已领取");
                }
                String remark = null;
                if ("tencent".equals(vipAccount.getCpid())) {
                    remark = "会员产品";
                }
                //20220909 添加 由于keep领取时，调下游接口必须要传递userid参数，故此处需要判断
                if ("keep".equals(vipAccount.getCpid())) {
                    remark = vipAccount.getUserid();
                }
                String orderid = vipAccount.getMonthtime() + "03" + vipAccount.getUserid() + "_" + vipAccount.getActivitytype() + "_" + vipAccount.getSpid();
                // 补送接口
                vipChangeUtil.regetVip(vipAccount, orderid, remark);
                String viptime = complementSendDao.findVipLogByRequestid(orderid);
                String curretime = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date());
                vipAccount.setViptime(viptime == null ? curretime : viptime);
                vipAccount.setViptype("3");  // 3 客服补送
                complementSendDao.updateAccount(vipAccount);
                vipGiveDao.saveVipGiveByMonth(vipAccount.getUserid(), vipAccount.getCpid(), vipAccount.getSpid(), vipAccount.getActivitytype(), vipAccount.getMonthtime());
                redisService.removeKey("sendvip_" + vipAccount.getActivitytype() + "_" + vipAccount.getCpid() + "_" + vipAccount.getSpid() + "_" + vipAccount.getUserid());
            } else {
                throw new BussinessException("9109", "补送会员配置异常，请联系管理员");
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.info("补送会员失败," + vipAccount.getUserid() + "====" + e.getMessage());
            throw e;
        }
    }

    /**
     * 获取补送vipid（区分合约产品与常规产品）
     *
     *
     * @param vipAccount
     * @return
     * @throws BussinessException
     */
    private String getRevipid(VipAccountBean vipAccount) throws BussinessException {
        String vipid;
        String revipid;
        String directype = complementSendDao.getDirectype(vipAccount.getSpid());
        //  取领取的vipid,用领取的vipid对应的revipid
        //  特殊的两款,未配置vipid  954   99133542
        if ("954".equals(vipAccount.getSpid())) {
            revipid = "20730";
        } else if ("99133542".equals(vipAccount.getSpid())) {
            revipid = "20738";
        } else {
            vipid = complementSendDao.getvipid(vipAccount.getSpid(), vipAccount.getActivitytype());
            if (!StringUtils.isEmpty(vipid)) {
                // 用领取的vipid指向的revipid
                revipid = complementSendDao.getRevipid(vipid);
                if (!StringUtils.isEmpty(directype) && "3".equals(directype)) {
                    // 合约包首月取领取的vipid对应的vipid，也就是revipid（通过逗号区分）的第一个
                    // 合约包
                    String[] vipids = revipid.split(",");
                    if (1 == vipids.length){
                        revipid = vipids[0];
                    }else {
                        if (!StringUtils.isEmpty(vipAccount.getIsfirstmonth()) && "1".equals(vipAccount.getIsfirstmonth())) {
                            revipid = vipids[0];
                        } else {
                            revipid = vipids[1];
                        }
                    }
                }
            } else {
                throw new BussinessException("9109", "补送会员配置异常，请联系管理员");
            }
        }
        return revipid;
    }

    /**
     * 领取后更新账期
     *
     * @param activityInfo
     * @param resultcode
     * @param msg
     * @param orderid
     */
    public void updateJoinAccount(VipInfo activityInfo, String resultcode, String msg, String orderid) {
        final VipAccountBean vipAccount = new VipAccountBean();
        vipAccount.setRequestid(orderid);
        if ("0".equals(resultcode) || StringUtils.isEmpty(resultcode)) {
            vipAccount.setViptype(activityInfo.getType() == 1 ? "1" : "2");
            vipAccount.setVipstatus("0");
            String viptime = complementSendDao.findVipLogByRequestid(orderid);
            String curretime = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date());
            vipAccount.setViptime(viptime == null ? curretime : viptime);
        } else {
            vipAccount.setErrorinfo(msg);
            vipAccount.setResultcode(resultcode);
        }
        vipAccount.setUserid(activityInfo.getUserid());
        vipAccount.setActivitytype(activityInfo.getActivetype());
        vipAccount.setSpid(activityInfo.getSpid());
        // TD_VIP_ACCOUNT保存requetid，用于更新结果
        complementSendDao.updateRequestid(vipAccount);
    }

}
