package com.chinacreator.video.service;

import com.chinacreator.video.bean.ChannelBean;
import com.chinacreator.video.bean.FlowQueryBean;
import com.chinacreator.video.bean.FlowUsedBean;
import com.chinacreator.video.bean.FlowUsedDetail;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.mapper.ChannelDao;
import com.chinacreator.video.mapper.CommConfDao;
import com.chinacreator.video.mapper.FlowDao;
import com.chinacreator.video.util.MD5;
import com.chinacreator.video.validate.IpValidate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

@Service
public class FlowService {

    @Autowired
    private ChannelDao channelDao;

    @Autowired
    private FlowDao flowDao;

    @Autowired
    private CommConfDao commConfDao;

    @Autowired
    private IpValidate ipValidate;

    @Autowired
    private ChannelValidate channelValidate;

    public FlowUsedBean findUsedFlow(FlowQueryBean flowQueryBean) throws Exception {

        this.checkBusi(flowQueryBean);

        FlowUsedBean flowUsedBean = flowDao.findUsedFlowBase(flowQueryBean);
        List<FlowUsedDetail> detail = flowDao.findUsedFlowDetail(flowQueryBean);
        flowUsedBean.setDetails(detail);

        return flowUsedBean;
    }

    public void checkBusi(FlowQueryBean flowQueryBean) throws Exception {
        this.checkQueryCondition(flowQueryBean);
        //检查时间戳
        this.valiTimestamp(flowQueryBean);
        channelValidate.validate(flowQueryBean.getChannel());

        ChannelBean channelBean = channelDao.queryChannel(flowQueryBean.getChannel());

        //检查IP
        this.ipValidate.validate(flowQueryBean.getChannel(), "1", null, null);

        this.valiSign(flowQueryBean, channelBean.getPassword());
    }

    private void checkQueryCondition(FlowQueryBean flowQueryBean) throws Exception {

        if (StringUtils.isEmpty(flowQueryBean.getStartdate())) {
            throw new BussinessException("10002", "开始时间不能为空");
        }

        if (StringUtils.isEmpty(flowQueryBean.getEnddate())) {
            throw new BussinessException("10002", "结束时间不能为空");
        }

        if (StringUtils.isEmpty(flowQueryBean.getTimestamp())) {
            throw new BussinessException("10002", "Timestamp不能为空");
        }

        if (StringUtils.isEmpty(flowQueryBean.getSign())) {
            throw new BussinessException("10002", "Sign不能为空");
        }

    }


    /**
     * 验证timestamp
     *
     * @throws Exception
     */
    private void valiTimestamp(FlowQueryBean flowQueryBean) throws Exception {
        if (StringUtils.isEmpty(flowQueryBean.getTimestamp())) {
            throw new BussinessException("10001", "timestamp不能为空");
        }

        //SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddhhmmss");
        //Date date = sdf.parse(flowQueryBean.getTimestamp());
        //long timetamp = date.getTime();
        long timetamp = Long.valueOf(flowQueryBean.getTimestamp());
        long localTimestamp = (System.currentTimeMillis());
        try {
            if ((localTimestamp - timetamp) > 1000 * 60 * 3 || (localTimestamp - timetamp) < -1000 * 60 * 3) {
                throw new BussinessException("10002", "timestamp已失效");
            }
        } catch (Exception e) {
            if (e instanceof BussinessException) {
                throw e;
            } else {
                throw new BussinessException("10002", "timestamp已失效");
            }
        }
    }

    /**
     * 签名验证
     *
     * @param flowQueryBean
     * @throws BussinessException
     */
    private void valiSign(FlowQueryBean flowQueryBean, String password) throws BussinessException {
        try {
            String localSign = getSign(flowQueryBean, password);

            if (!localSign.equals(flowQueryBean.getSign())) {
                throw new BussinessException("10006", "签名校验失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new BussinessException("10006", "签名校验失败");
        }
    }

    /**
     * 拼接签名信息 channel + cpid + spid + userid + startdate + enddate + size + page + timestamp + password
     *
     * @param flowQueryBean
     * @return
     */
    private String getSign(FlowQueryBean flowQueryBean, String password) throws Exception {
        if (flowQueryBean == null) {
            return "";
        }

        //channel,cpid, spid, userid, startdate, enddate, size, page, timestamp,
        //signature = DesUtil.encode(channel + cpid + enddate + page + size + spid + startdate + userid + timestamp, password);

        StringBuilder sb = new StringBuilder();
        sb.append(flowQueryBean.getChannel())
                .append(flowQueryBean.getCpid() == null ? "" : flowQueryBean.getCpid())
                .append(flowQueryBean.getEnddate() == null ? "" : flowQueryBean.getEnddate())
                .append(flowQueryBean.getPage() == null ? "" : flowQueryBean.getPage())
                .append(flowQueryBean.getSize() == null ? "" : flowQueryBean.getSize())
                .append(flowQueryBean.getSpid() == null ? "" : flowQueryBean.getSpid())
                .append(flowQueryBean.getStartdate() == null ? "" : flowQueryBean.getStartdate())
                .append(flowQueryBean.getTimestamp() == null ? "" : flowQueryBean.getTimestamp())
                .append(flowQueryBean.getUserid() == null ? "" : flowQueryBean.getUserid())
                .append(password);

        System.out.println("sign=" + sb.toString());
        return MD5.MD5Encode(sb.toString());
    }

    public static void main(String[] args) throws ParseException {
        Map<String, String> map = new LinkedHashMap<String, String>() {{
            put("channel", "");
            put("cpid", "");
            put("spid", "");
            put("userid", "");
            put("startdate", "");
            put("enddate", "");
            put("size", "");
            put("page", "");
            put("timestamp", "");
        }};

        for (String key : map.keySet()) {
            System.out.println(key + ",");
        }

        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddhhmmss");
        Date date = sdf.parse("20190724010101");

        System.out.println(date.getTime());
    }

}
