package com.chinacreator.video.service;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.video.bean.VipInfo;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.log.VipLog;
import com.chinacreator.video.mapper.SysDao;
import com.chinacreator.video.mapper.VipDao;
import com.chinacreator.video.util.MD5;
import com.chinacreator.video.util.TimeUtil;
import com.chinacreator.video.util.URLUtil;

/**
 * 
 * PPTV会员领取
 * 
 * @author yanlong
 *
 */
@Service
public class PptvGiveVipService {
	private final static String CHANNEL = "unicom_broadband";
	private final static String SECRET_KEY = "AXoWaXkQCyUE";
    private final static String PACKGE_31 = "pp31d";
    
//	private final static String BASE_URL = "http://uc.test.pptv.com/cusp/vipadd/corpadd";   //TEST
    private final static String BASE_URL = "http://billing.api.pptv.com/cusp/vipadd/corpadd";   //PROD
    
    private static final Logger log = LoggerFactory.getLogger("viplog");
    
    @Autowired
    private SysDao sysDao;
    @Autowired
    private VipDao vipDao;
    @Autowired
    VipControlService vipControlService;

    @Autowired
    ComplementSendService complementSendService;

    public void giveVip(VipInfo activityInfo) throws BussinessException {
        this.vipControlService.canGiveVip(activityInfo);

        VipLog vipLog = new VipLog();
        vipLog.setResultcode("0");
        vipLog.setErrorinfo("");
        vipLog.setActivityType(activityInfo.getActivetype());
        vipLog.setUserid(activityInfo.getUserid());
        vipLog.setType(activityInfo.getType());
        vipLog.setUsertype(activityInfo.getUsertype());
        vipLog.setUserval(activityInfo.getUserval());
        vipLog.setVipdays("31");
        
      //订单重试
        boolean flagCorp=false;
        String toCorpOrderid=null;
        if(activityInfo.getVippool()!=null && !"".equals(activityInfo.getVippool().trim())){
        	toCorpOrderid =  vipDao.getOrderId2(activityInfo.getUserid(), activityInfo.getActivetype(),activityInfo.getVippool());
        }else{
        	toCorpOrderid =  vipDao.getOrderId(activityInfo.getUserid(), activityInfo.getActivetype());
        }
        if(toCorpOrderid != null && !"".equals(toCorpOrderid)){
        	flagCorp=true;
        }

        try {
            Map<String, String> param = new HashMap<String,String>();
            
			String packageid = PACKGE_31;
			vipLog.setVipdays("31");
			
			//重试订单
            String order_id = null;
            if(flagCorp){
            	order_id = toCorpOrderid;
            }else{
            	order_id = this.sysDao.getId();
            }

			String begintime = this.sysDao.getSysTime();
			String endtime = this.sysDao.getLastDay();

			vipLog.setOrderid(order_id);
			vipLog.setBegintime(begintime);
			vipLog.setEndtime(endtime);

			String phone  = activityInfo.getUserval();
			String orderTime = TimeUtil.dateFormat(new Date(),"yyyyMMddHHmmss");
			String format = "json";
			//签名MD5加密
			String sign = MD5.MD5Encode(phone + "&" + CHANNEL + "&" + order_id + "&" + SECRET_KEY);

			//合作方直充接口参数
			param.put("phone",phone);
			param.put("channel",CHANNEL);
			param.put("status","1");	//订购
			param.put("packageid",packageid);
			param.put("outorderid",order_id);
			param.put("format",format);
			param.put("orderTime",orderTime);
			param.put("sign",sign);

			String url = URLUtil.mapToUrl(BASE_URL,param);

			String result=null;
			try {
				vipLog.setOutput(JSON.toJSONString(param));
                result = URLUtil.get(url);
//				result = "{\"message\":\"success\",\"errorCode\":\"0\"}";
				vipLog.setReturnput(result);
			} catch (Exception e) {
				vipLog.setResultcode("9070");
				vipLog.setErrorinfo("赠送会员超时或异常");
				throw new BussinessException("9070", "赠送会员超时或异常");
			}
			JSONObject obj = JSON.parseObject(result);
			
			//解析返回结果
			String msg = obj.getString("message");
			String code = obj.getString("errorCode");
			
			//订单是否成功
			boolean flagvip = false;
			if("0".equals(code)){
				flagvip=true;
			}
			
			try {
				//把原有的9070订单标记失败
				if(flagCorp && flagvip && vipDao.getCount9070(activityInfo.getUserid(), toCorpOrderid)>0){
					vipDao.updateOrderIdAll(activityInfo.getUserid(), toCorpOrderid, "1","订单重试表明：此订单失败");
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
			
			vipLog.setResultcode(flagvip?"0":code);
			vipLog.setErrorinfo(msg);

			complementSendService.updateJoinAccount(activityInfo,vipLog.getResultcode(),msg,order_id);
			this.vipControlService.saveVipGive(activityInfo, vipLog.getResultcode());
			if (!flagvip) {
				throw new BussinessException("9071", "赠送会员失败(" + obj.getString("message") + ")");
			}
        }catch (Exception e){
            if (e instanceof BussinessException)
                throw (BussinessException) e;
            else
                throw new BussinessException("8000",e.getMessage());
        }finally{
            log.info(JSON.toJSONString(vipLog));
            saveLog(vipLog);
        }
    }

    public void saveLog(VipLog vipLog){
//        vipLog.setResultcode("1");
    	
    	if(vipLog.getReturnput()==null){
			vipLog.setReturnput("");
		}
		if(vipLog.getResultcode()==null){
			vipLog.setResultcode("-0");
		}
		if(vipLog.getErrorinfo()==null){
			vipLog.setErrorinfo("");
		}
        this.vipDao.insert(vipLog);
    }

}
