package com.chinacreator.video.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.video.bean.OrderInfo;
import com.chinacreator.video.bean.SPInfo;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.mapper.OrderDao;
import com.chinacreator.video.mapper.SpDao;
import com.chinacreator.video.pojo.OrderBean;
import com.chinacreator.video.pojo.Product;
import com.chinacreator.video.pojo.SpProduct;
import com.chinacreator.video.util.SequenceUtils;
import com.chinacreator.video.util.TimeUtil;
import com.chinacreator.video.util.URLUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.sql.SQLException;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * Create by IntelliJ IDEA.
 * Author: EricJin
 * Date: 08/14/2018 15:35
 */
@Service
public class UnifiedAuthService {

    @Autowired
    private SpDao spDao;

    @Autowired
    private OrderDao orderDao;

    private static final Logger LOGGER = LoggerFactory.getLogger("unified");

//    private static final String AUTH_URL = "http://10.199.99.177:86/unifiedauth/ordersync";   //TEST
    private static final String AUTH_URL = "http://10.199.99.51:809/unifiedauth/ordersync";   //PROD

//        private static final String AUTH_REMAINSYCN_URL = "http://10.199.99.177:86/unifiedauth/remainUserSync";   //TEST
    private static final String AUTH_REMAINSYCN_URL = "http://10.199.99.51:809/unifiedauth/remainUserSync";   //PROD

    /**
     * call the unifiedAuth service to Sycn the Order to the unifiedAuth platform
     * @param orderInfo
     * @return
     * @throws Exception
     */
    public Boolean sycnOrder(OrderInfo orderInfo) throws Exception {
        SPInfo spInfo = spDao.findById(orderInfo.getSpid());
        //20190402，xu.zhou增加seqnumber不为空判断（不为空就不走同步验证了）
        if(!StringUtils.isEmpty(orderInfo.getSeqnumber())){
        	return true;
        }

        //not need sycn Order, return true to go to order
        if(!checkNeededSycnOrder(orderInfo,spInfo)) return true;

        //if need to sycn Order , assemble the Bean and post to the unifiedAuth
        OrderBean orderBean = assembleOrderBean(orderInfo,spInfo);
        //wrap the orderBean to json
        ObjectMapper mapper = new ObjectMapper();
        String json = mapper.writeValueAsString(orderBean);
        LOGGER.info("【AuthRequ====】" + json);
        String resp = URLUtil.post(AUTH_URL, json,10000,"application/json");
        LOGGER.info("【AuthResp----】" + resp);
        //转换json to JSONObject
        JSONObject obj = JSON.parseObject(resp);

        //get the response,and check the code.
         if ("0000".equals(obj.getString("resultcode"))) return true;
         else throw new BussinessException(obj.getString("resultcode"), "UnifiedAuth Failed:" + obj.getString("resultinfo"));
    }

    private boolean checkNeededSycnOrder(OrderInfo orderInfo, SPInfo spInfo) throws Exception {
        //not need return false
        if (spInfo.getNeedunifiedauthsync() == null || !"1".equals(spInfo.getNeedunifiedauthsync().trim())) return false;
        //need ,then check the token
        if (StringUtils.isEmpty(orderInfo.getToken())) {
            throw new BussinessException("8002", "TOKEN参数无效");
        }
        return true;
    }

    private OrderBean assembleOrderBean(OrderInfo orderInfo, SPInfo spInfo) throws SQLException {

        String sequence = TimeUtil.dateFormat(new Date(),"MMddhhmmss") + "420010" + SequenceUtils.getSequence();
        orderInfo.setId(sequence);

        OrderBean orderBean = new OrderBean();
        orderBean.setSeqNumber(sequence);       //订单流水号
        orderBean.setAccessType(orderInfo.getAccess());   //h5, sdk
        orderBean.setToken(orderInfo.getToken());
        orderBean.setOrderId(sequence);
        orderBean.setPhone(orderInfo.getUserid());
        orderBean.setPrice(spInfo.getPrice());

        if ("1".equals(orderBean.getAccessType())) orderBean.setOrderMethod("1");   //web
        else if ("2".equals(orderBean.getAccessType())) orderBean.setOrderMethod("5");  //app

        orderBean.setFeeplan(spInfo.getType() == 0 ? "3" : "1");        //0 按次--3 包周期  其他包月--1
        orderBean.setUpdateType("1");       //1 订购  2 续订
        orderBean.setSubscribeType("1");
        orderBean.setServiceType("80");

        //set product
        SpProduct spProduct = new SpProduct(spInfo.getSpid(),spInfo.getSpname());
        orderBean.getSpProduct().add(spProduct);
        Product product = new Product(spInfo.getVacproductid(),spInfo.getSpname());
        orderBean.getProduct().add(product);

        orderBean.setSpid(spInfo.getSpcode());
        orderBean.setSpName(spInfo.getSpcodename());

        orderBean.setDeveloperName("联通在线信息科技有限公司");
        orderBean.setSpServPhone("10010");   //sp客服电话
        setDate(orderBean);
        return orderBean;
    }

    private void setDate(OrderBean orderBean) {

        String format = "yyyyMMddHHmmss";
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        String now = TimeUtil.getDateFromFormat(format,calendar);

        //set start date
        orderBean.setSubscribeTime(now);
        orderBean.setRentDate(now);
        orderBean.setEffectiveDate(now);

        //set end date
        calendar.add(Calendar.MONTH,1);
        calendar.set(Calendar.HOUR,23);
        calendar.set(Calendar.MINUTE,59);
        calendar.set(Calendar.SECOND,59);
        String expireDate = TimeUtil.getDateFromFormat(format,calendar);
        orderBean.setExpireDate(expireDate);
    }


    /**
     * 续订同步
     * @param orderInfo
     * @throws BussinessException
     */
    public void remainUserSycn(OrderInfo orderInfo) throws BussinessException {
        //判断是否需要同步
        SPInfo spInfo = spDao.findById(orderInfo.getSpid());
        if (spInfo.getNeedunifiedauthsync() == null || !"1".equals(spInfo.getNeedunifiedauthsync().trim())) return;

        String sequence = TimeUtil.dateFormat(new Date(),"MMddhhmmss") + "420010" + SequenceUtils.getSequence();
        Map<String, String> params = new HashMap<>();
        params.put("userid",orderInfo.getUserid());
        params.put("spid",orderInfo.getSpid());
        params.put("cpid",orderInfo.getCpid());
        params.put("seq",sequence);     //同步此次的sequenceNumber
        String url = URLUtil.mapToUrl(AUTH_REMAINSYCN_URL,params);
        String resp = null;
        try {
            resp = URLUtil.get(url);
        } catch (Exception e) {
            throw new BussinessException("10024","调用订单同步接口超时");
        }
        params.put("response",resp);
        LOGGER.info("【remainUserSycn】" + JSONObject.toJSONString(params));
        JSONObject obj = JSONObject.parseObject(resp);
        String resultcode = obj.getString("resultcode");
        String resultinfo = obj.getString("resultinfo");
        String orderid = obj.getString("seqnumber");

        //设置首订同步订单号  设置VAC同步sequenceNumber
        orderInfo.setId(sequence);
        orderInfo.setAuthsycnorderid(orderid);
        //get the response,and check the code.
        if (!"0000".equals(resultcode))
            throw new BussinessException(resultcode, "UnifiedAuth Failed:" + resultinfo);
//        //保存orderid
//        if (!StringUtils.isEmpty(orderid))
//            orderDao.updateSycnOrderId(orderInfo);
    }

}
