package com.chinacreator.video.service;


import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSON;
import com.chinacreator.video.bean.UserMsgBean;
import com.chinacreator.video.bean.VipInfo;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.mapper.ActiveMango3Dao;
import com.chinacreator.video.mapper.UserMsgDao;
import com.chinacreator.video.redis.RedisService;
import com.chinacreator.video.util.DesUtil;
import com.chinacreator.video.util.SHAUtil;
import com.chinacreator.video.util.URLUtil;
import com.chinacreator.video.util.VipChangeUtil;

@Component
public class VipMango3Service {
	
	private static Logger ErrorLOGGER = LoggerFactory.getLogger("ERROR");
	
	//只能领取此活动会员
	private static String  activetype = "29";
	
	//redis计数key前缀
	private static String key = "joinvip_2020mango_3_";
	
	private static String vipid="21591";
	private static String vippwd="ts9sng9d";
	private static String orderid_p = "2020mango3_";
	
	private static int setDay = 400;
	private static int setTotal = 8000;
	
	@Autowired
	private RedisService redisService;
	
	@Autowired
	private ComonService comonService;

	@Autowired
	private MobService mobService;

	@Autowired
	private ActivityConfigService activityService;
	
	@Autowired
	private ActiveMango3Dao activeDao;
	@Autowired
	VipChangeUtil vipChangeUtil;
//	@Autowired
//	private UserMsgDao userMsgDao;

	public void addRedis(VipInfo vipInfo) throws BussinessException {
		try {
			String key1 = key+new SimpleDateFormat("yyyyMMdd").format(new Date());
			long count = redisService.incrAdd(key1,60L*60L*24L*2L);
			System.out.println(key1+":"+count);
			
			String key2 = key+"total";
			long count2 = redisService.incrAdd(key2,60L*60L*24L*365L);
			System.out.println(key2+":"+count2);
		} catch (Exception e) {
			ErrorLOGGER.error("", e);  
		}
	}
	
	/**
	 * 并发请求
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkRedis(VipInfo vipInfo) throws BussinessException {
		try {
			long count = redisService.incr("joinvip_" + vipInfo.getActivetype() + "_" + vipInfo.getUserid());
			if (count > 1) {
				throw new BussinessException("8001", "并发请求，请稍后再试");
			}
		} catch (Exception e) {
			if (e instanceof BussinessException) {
				throw e;
			}else{
				ErrorLOGGER.error("", e);  
			}
		}
	}
	
	
	/**
	 * 限时限量校验（1、每天限量，2、总限量）
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkRedisCount(VipInfo vipInfo) throws BussinessException {
		//每天10点以后才能领取
		Calendar cal = Calendar.getInstance();
		int hour = cal.get(Calendar.HOUR_OF_DAY);
		if(hour < 10) {
			throw new BussinessException("8701", "本次活动上午10点才开始，请上午10点再来领取");
		}
		
		//每天限量300份
		String countDay = redisService.get(key+new SimpleDateFormat("yyyyMMdd").format(new Date()));
		if(countDay != null) {
			int count = 0;
			try {
				count = Integer.parseInt(countDay);
			} catch (Exception e) {}
			if(count >= setDay ) {
				throw new BussinessException("8702", "本次活动当天已结束，请次日上午10点再来领取");
			}
		}
		
		int curDay = activeDao.queryVipCountCurDay();
		if(curDay >= setDay) {
			throw new BussinessException("8702", "本次活动当天已结束，请次日上午10点再来领取");
		}
		
		//总量限量8000份
		String countTotal = redisService.get(key+"total");
		if(countTotal != null) {
			int count = 0;
			try {
				count = Integer.parseInt(countTotal);
			} catch (Exception e) {}
			if(count >= setTotal ) {
				throw new BussinessException("8703", "本次活动已结束，请期待下次活动");
			}
		}
		
		int total = activeDao.queryVipCountTotal();
		if(total >= setTotal) {
			throw new BussinessException("8703", "本次活动已结束，请期待下次活动");
		}
	}
	
	/**
	 * 判断活动领取资格（1、未订购芒果16元前后向产品，2、联通用户）
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkJoinActive(VipInfo vipInfo) throws BussinessException {
		try {
			int count = activeDao.queryOrderRelation16(vipInfo.getUserid());
			if(count>0) {
				throw new BussinessException("8704", "您不在本次活动范围内，请期待下次活动");
			}
			
			/**
			 * 查询号段
			 * 
			 */
			int i = activeDao.queryHeader(vipInfo.getUserid().substring(0, 7));
			if(i<1){
				int j = activeDao.queryXHZW(vipInfo.getUserid());
				if(j<1) {
					throw new BussinessException("8705", "您不在本次活动范围内，请期待下次活动");
				}
			}
		} catch (Exception e) {
			if (e instanceof BussinessException) {
				throw (BussinessException)e;
			}else{
				ErrorLOGGER.error("", e);  
				throw e;
			}
		}
		
	}
	
	/**
	 * 校验是否已经领取
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkJoinVip(VipInfo vipInfo) throws BussinessException {
		
		int joincount = activeDao.getJoinRecord(vipInfo.getUserid());
		if(joincount > 0) {
			throw new BussinessException("8706", "您已经参与过本次活动，不能重复参与");
		}
		
	}
	
	
	/**
	 * 活动相关信息校验
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkMango3(VipInfo vipInfo) throws BussinessException {
		if(!activetype.equals(vipInfo.getActivetype())){
			throw new BussinessException("9083", "无效活动");
		}
	}
	
//	public void totalCount(VipInfo vipInfo) throws BussinessException {
//		String key1 = key+new SimpleDateFormat("yyyyMMdd").format(new Date());
//		
//		String countDay = redisService.get(key+new SimpleDateFormat("yyyyMMdd").format(new Date()));
//		int curCount = activeDao.queryVipCountCurDay();
//		if(countDay == null) {
//			long count = redisService.incrAdd(key1,60L*60L*24L*2L);
//			
//		}
//		
//		
//		String countTotal = redisService.get(key+"total");
//		int totalCount = activeDao.queryVipCountTotal();
//		if(countTotal == null) {
//			countTotal = "0";
//		}
//		
//		
////		String key1 = key+new SimpleDateFormat("yyyyMMdd").format(new Date());
////		long count = redisService.incrAdd(key1,60L*60L*24L*2L);
////		System.out.println(key1+":"+count);
////		
////		String key2 = key+"total";
////		long count2 = redisService.incrAdd(key2,60L*60L*24L*365L);
////		System.out.println(key2+":"+count2);
//		
//		
////		Map<String,String > map =new HashMap<String, String>();
////		
////		
////		
////		map.put("curday", countDay);
////		map.put("curtotal", countTotal);
////		
////		map.put("setday", setDay+"");
////		map.put("settotal", setTotal+"");
//		
//	}
	
	
	public Map<String,String> queryCount(VipInfo vipInfo) throws BussinessException {
		Map<String,String > map =new HashMap<String, String>();
		
		String countDay = redisService.get(key+new SimpleDateFormat("yyyyMMdd").format(new Date()));
		if(countDay == null) {
			countDay = "0";
		}
		
		String countTotal = redisService.get(key+"total");
		if(countTotal == null) {
			countTotal = "0";
		}
		
		int curDay = activeDao.queryVipCountCurDay();
		if(curDay>Integer.parseInt(countDay)) {
			countDay = curDay+"";
		}
		
		int total = activeDao.queryVipCountTotal();
		if(total>Integer.parseInt(countTotal)) {
			countTotal = total+"";
		}
		
		map.put("curday", countDay);
		map.put("curtotal", countTotal);
		
		map.put("setday", setDay+"");
		map.put("settotal", setTotal+"");
		
		return map;
	}
	
	
	
	/**
	 * 刷新缓存
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void refresh(VipInfo vipInfo) throws BussinessException {
		this.checkMango3(vipInfo);//判断活动代码
		comonService.checkTimestamp(vipInfo.getTimestamp());
		activityService.checkActivity(vipInfo);
	}
	
	/**
	 * 领取资格查询校验
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkVipQuery(VipInfo vipInfo) throws BussinessException {
		this.checkMango3(vipInfo);//判断活动代码
		comonService.checkTimestamp(vipInfo.getTimestamp());
		activityService.checkActivity(vipInfo);
		comonService.checkQueryVipSign(vipInfo);
		mobService.checkVipMob(vipInfo);
		
		this.checkRedisCount(vipInfo);
		this.checkJoinActive(vipInfo);
		this.checkJoinVip(vipInfo);
	}
	
	/**
	 * 领取校验
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkVip(VipInfo vipInfo) throws BussinessException {
		this.checkRedis(vipInfo);//并发请求
		this.checkMango3(vipInfo);//判断活动代码
		comonService.checkTimestamp(vipInfo.getTimestamp());
		activityService.checkActivity(vipInfo);
		comonService.checkQueryVipSign(vipInfo);
		mobService.checkVipMob(vipInfo);
		
		this.checkRedisCount(vipInfo);
		this.checkJoinActive(vipInfo);
		this.checkJoinVip(vipInfo);
	}
	
	/**
	 * 会员领取
	 * 
	 * @param activityInfo
	 * @throws BussinessException
	 */
	public void giveVip(VipInfo activityInfo) throws BussinessException {
		String orderid = orderid_p+activityInfo.getUserid();
		
		/**
		 *  保存会员领取
		 */
		int i=activeDao.insert(activityInfo.getUserid(), activityInfo.getActivetype(), vipid, "1", orderid);
		if(i>0) {
			
			try {
				/**
				 * redis计数
				 */
				this.addRedis(activityInfo);
				
				//领取会员
				vipChangeUtil.getVip(vipid, activityInfo.getUserid(), activityInfo.getActivetype(), vippwd, "4", activityInfo.getUserid(), orderid, null ,activityInfo.getType()+"");
			
			} catch (BussinessException e) {
				String code = e.getCode();
				
				activeDao.deteleJoining(activityInfo.getUserid());
				
				if("9060".equals(code)) {
					throw new BussinessException("8708", "活动会员领取达到限额");
				}else {
					throw new BussinessException("8707", "会员领取失败，请稍后再试");
				}
			}
			
			activeDao.joinSucceed(activityInfo.getUserid(), "0");
			
//			/**
//			 *保存领取账号
//			 */
//			userMsgDao.insertMsg(getUserMsgBean(activityInfo));
			
			/**
			 * 发送短信
			 */
			try {
				int count15 = activeDao.queryOrderRelation15(activityInfo.getUserid());
				if(count15<=0) {
					String smsid="5003102";
					String pwd= "soews92d";
					String url ="http://111.206.133.54/smsbusi/sms/send";
					String content = "【中国联通】"+activityInfo.getUserid()+"，您已成功领取3天芒果TV VIP会员。现在订购wo+视频芒果TV定向流量包月，还可免流看视频，快来点击订购吧  https://v.wo.cn/mg15hd20";
					this.sendSMS(url, smsid, pwd, activityInfo.getUserid(), content);
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
			
		}else {
			throw new BussinessException("8707", "会员领取失败，请稍后再试");
		}
	}
	
//	public UserMsgBean getUserMsgBean(VipInfo activityInfo) {
//		UserMsgBean userMsgBean = new UserMsgBean();
//		userMsgBean.setActiveid(activityInfo.getActivetype());
//		userMsgBean.setCpid(activityInfo.getCpid());
//		userMsgBean.setMsg1(activityInfo.getUserval());
//		userMsgBean.setMsg2(activityInfo.getUsertype());
//		userMsgBean.setMsg3(activityInfo.getVipname() == null ?"":activityInfo.getVipname());
//		userMsgBean.setSpid(activityInfo.getSpid());
//		userMsgBean.setUserid(activityInfo.getUserid());
//		System.out.println("UserMsgBean="+JSON.toJSONString(activityInfo));
//		return userMsgBean;
//	}
	
	
	/**
	 * 发送短信
	 * 
	 * @param usermob
	 * @param content
	 * @return
	 */
	private String sendSMS (String dealurl,String smsid,String key, String usermob, String content){
		try {
			String timestamp = System.currentTimeMillis()/1000+"";
			String userid =  DesUtil.encode(usermob,  key);
			String sign = SHAUtil.shaEncode(smsid+userid+timestamp+content+key).toLowerCase();
			
			userid = URLEncoder.encode(userid,"utf-8");
			content = URLEncoder.encode(content,"utf-8");
			String url = "?userid="+userid+"&smsid="+smsid+"&timestamp="+timestamp+"&sign="+sign+"&content="+content;
			String resp = URLUtil.get(dealurl+url);
			
			System.out.println(url+","+resp);
			return resp;
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return null;
	}
	
}
