package com.chinacreator.video.service.vip;


import org.apache.commons.lang.math.NumberUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.video.bean.HxOrder;
import com.chinacreator.video.bean.HxProduct;
import com.chinacreator.video.bean.HxVipInfo;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.mapper.HxOrderDao;
import com.chinacreator.video.mapper.VipDao;
import com.chinacreator.video.service.ComonService;
import com.chinacreator.video.util.AESUtil;
import com.chinacreator.video.util.DesUtil;
import com.chinacreator.video.util.HxVipUtil;
import com.chinacreator.video.util.MD5;
import com.chinacreator.video.util.UsermobUtil;

@Component
public class HxOrderService {
	
	private static final String MOB_KEY = "ac22273abb2f4960";
	
	@Autowired
	private HxOrderDao hxDao;
	
	@Autowired
	private VipDao vipDao;

	@Autowired
	private ComonService comonService;

	@Autowired
	private HxVipUtil hxVipUtil;
	/**
	 * Sign校验
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkSign(HxVipInfo vipInfo) throws BussinessException {
		if (StringUtils.isEmpty(vipInfo.getOrderid())) {
			throw new BussinessException("9054", "orderid不能为空");
		}
		if (StringUtils.isEmpty(vipInfo.getUserid())) {
			throw new BussinessException("9054", "userid不能为空");
		}
		if (StringUtils.isEmpty(vipInfo.getCpid())) {
			throw new BussinessException("9054", "cpid不能为空");
		}
		if (StringUtils.isEmpty(vipInfo.getSpid())) {
			throw new BussinessException("9054", "spid不能为空");
		}
		String sign = vipInfo.getSignature();
		if (StringUtils.isEmpty(sign)) {
			throw new BussinessException("9054", "signature不能为空");
		}
		HxProduct goods = hxDao.getProdcutBean(vipInfo.getCpid(), vipInfo.getSpid());
		String pwd = goods.getPwd();
		String data = vipInfo.getOrderid() + vipInfo.getUserid() + vipInfo.getTimestamp() + pwd;
		String localSign = MD5.MD5Encode(data);
		System.out.println(">>>>>>>>>>>>>>>>用户上传的sign:"+sign);
		System.out.println(">>>>>>>>>>>>>>>>服务生成的sign:"+localSign);
		if (!localSign.equals(sign)) {
			throw new BussinessException("9054", "signature校验失败");
		}
		
		vipInfo.setPwd(pwd);
		System.out.println(">>>>>>>>>>>>>>>>密码:"+vipInfo.getPwd());
		
//		vipInfo.setViptype(goods.getViptype());
//		vipInfo.setVippool(goods.getVippool());
		vipInfo.setVipid(goods.getVipid());
		vipInfo.setVipchannel(goods.getVipchannel());
		vipInfo.setVippwd(goods.getVippwd());
		
	}
	
	/**
	 * userid校验与解密
	 * 
	 * @param activityInfo
	 * @throws BussinessException
	 */
	public void checkMob(HxVipInfo vipInfo) throws BussinessException {
		String userid = vipInfo.getUserid();
		if(StringUtils.isEmpty(userid)) {
			throw new BussinessException("9054", "userid不能为空");
		}
		try {
			userid = DesUtil.decode(userid, vipInfo.getPwd());
			if(userid.length() > 11) {
				userid = AESUtil.decrypt(userid, MOB_KEY);
			}
			if(!UsermobUtil.isValid(userid)) {
				if(userid.length()>10 ||  userid.length()<5) {
					throw new BussinessException("9055", "userid不正确");
				}
			}
		}catch(Exception e) {
			throw new BussinessException("9056", "userid加密错误");
		}
		vipInfo.setUserid(userid);
	}
	
	public void checkOrderID(HxVipInfo vipInfo) throws BussinessException {
		if(StringUtils.isEmpty(vipInfo.getOrderid())) {
			throw new BussinessException("9054", "orderid不能为空");
		}
		int num;
		try {
			num=NumberUtils.toInt(vipDao.getRequestId(vipInfo.getVipid(),vipInfo.getOrderid()));
		}catch(Exception e) {
			throw new BussinessException("9070","赠送会员超时或异常。");
		}
		if(num > 0) {
			throw new BussinessException("0", "");
		}else {
			int num2=0;
			try {
				num2=NumberUtils.toInt(vipDao.getRequestIdFail(vipInfo.getVipid(),vipInfo.getOrderid()));
			}catch(Exception e) {
				throw new BussinessException("9070","赠送会员超时或异常。");
			}
			if(num2 > 0) {
				throw new BussinessException("9071", "会员权益充值失败");	
			}
		}
	}
	
	
	/**
	 * 请求校验
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkVip(HxVipInfo vipInfo) throws BussinessException {
//		this.checkRedis(vipInfo);
		comonService.checkTimestamp(vipInfo.getTimestamp());
		this.checkSign(vipInfo);
		this.checkMob(vipInfo);
		this.checkOrderID(vipInfo);
	}
	
	
	
	/**
	 * 会员领取
	 * 
	 * @param info
	 * @throws BussinessException
	 */
	public void giveVip(HxVipInfo info) throws BussinessException {
		try {

			HxOrder bean = hxDao.getOrderBean(info.getUserid(), info.getOrderid(), info.getCpid(), info.getSpid());
			if(bean == null ) {
				throw new BussinessException("3005", "此订单不存在");
			}
			
			//赠送中
			if(!"2".equals(bean.getVipstatus())){
//				throw new BussinessException("3006", "此订单不在[会员领取中]状态");
				throw new BussinessException("9070","赠送会员超时或异常。");
			}
			
		} catch (Exception e) {
			e.printStackTrace();
			
			if (e instanceof BussinessException)
				throw (BussinessException) e;
			else
//				throw new BussinessException("3007", "查询订单异常");
			    throw new BussinessException("9070","赠送会员超时或异常。");
		}
		
		//调用直充接口送会员
		try {
			String remark = null;
			if("tencent".equals(info.getCpid())){
				remark = "会员产品";
			}
			
			String result = hxVipUtil.vipid(info.getVipid(), info.getVipchannel(), info.getVippwd(), info.getUsertype(), info.getUserval(), info.getOrderid(),remark);
			if(result == null || "".equals(result.trim())) {
				throw new BussinessException("9070","赠送会员超时或异常。");
			}
			JSONObject obj = JSON.parseObject(result);
			String resultcode = obj.getString("resultcode");
			if(resultcode == null || "".equals(resultcode.trim()) || "8000".equals(resultcode.trim())) {
				throw new BussinessException("9070","赠送会员超时或异常。");
			}
			String errorinfo = obj.getString("errorinfo");
			if(!"0".equals(resultcode) && !("10002".equals(resultcode) && "orderid不能重复使用".equals(errorinfo)) ){
//				if(!"9070".equals(resultcode) && !"10021".equals(resultcode) && !"10005".equals(resultcode)){
//					//会员失败
//					hxDao.giveFail(info.getUserid(),info.getOrderid(), info.getCpid(), info.getSpid());
//				}
				
				throw new BussinessException("9070","赠送会员超时或异常。");
			}
		} catch (Exception e) {
			e.printStackTrace();
			
			throw new BussinessException("9070","赠送会员超时或异常。");
		}
		
		//会员成功
//		hxDao.giveSuccess(info.getUserid(),info.getOrderid(), info.getCpid(), info.getSpid());
	}
	
	
	
}
