package com.chinacreator.video.service.vip;


import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.math.NumberUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.video.bean.ActivityConfigBean;
import com.chinacreator.video.bean.CpSpBean;
import com.chinacreator.video.bean.HxOrder;
import com.chinacreator.video.bean.HxProduct;
import com.chinacreator.video.bean.HxVipInfo;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.mapper.ActivityConfigDao;
import com.chinacreator.video.mapper.CpSpDao;
import com.chinacreator.video.mapper.HxOrderDao;
import com.chinacreator.video.mapper.SysDao;
import com.chinacreator.video.mapper.VipDao;
import com.chinacreator.video.redis.RedisService;
import com.chinacreator.video.service.ComonService;
import com.chinacreator.video.util.DesUtil;
import com.chinacreator.video.util.HxVipTwUtil;
import com.chinacreator.video.util.MD5;
import com.chinacreator.video.util.UsermobUtil;

@Component
public class HxOrderTwService {
	
	@Autowired
	private RedisService redisService;
	
	@Autowired
	private ComonService comonService;
	
	@Autowired
	private ActivityConfigDao activityConfigDao;
	
	@Autowired
	private SysDao sysdao;
	
	@Autowired
	private CpSpDao CpSpDao;
	
	@Autowired
	private VipDao vipDao;
	
	@Autowired
	private HxOrderDao hxDao;

	@Autowired
	private HxVipTwUtil hxVipTwUtil;
	
	/**
	 * 并发控制
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	private void checkRedis(HxVipInfo vipInfo) throws BussinessException {
		long count = redisService.incr("hxvip_" + vipInfo.getCpid()+ "_" + vipInfo.getSpid() + "_" + vipInfo.getUserid() + "_" + vipInfo.getOrderid());
		if (count > 1) {
			throw new BussinessException("8001", "并发请求，请稍后再试");
		}
	}
	
	/**
	 * 验证活动代码
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	private void checkActivity(HxVipInfo vipInfo) throws BussinessException {
		if(StringUtils.isEmpty(vipInfo.getActivetype())) {
			throw new BussinessException("9083", "Activetype无效");
		}
		if(!vipInfo.getActivetype().startsWith("9")){
			throw new BussinessException("9083", "Activetype无效");
		}

		ActivityConfigBean activityConfigBean = activityConfigDao.getConfig(vipInfo.getActivetype());
		if(activityConfigBean == null || !"0".equals(activityConfigBean.getStatus()) || NumberUtils.toLong(sysdao.getSysTime())>NumberUtils.toLong(activityConfigBean.getEndtime()) || NumberUtils.toLong(sysdao.getSysTime())<NumberUtils.toLong(activityConfigBean.getBegintime())) {
			throw new BussinessException("9083", "该活动已失效");
		}
		vipInfo.setCpid(activityConfigBean.getCpid());
		vipInfo.setSpid(activityConfigBean.getSpid());
	}

	/**
	 * Sign校验
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	private void checkSignQuery(HxVipInfo vipInfo) throws BussinessException {
		if (StringUtils.isEmpty(vipInfo.getUserid())) {
			throw new BussinessException("9054", "userid不能为空");
		}
		if (StringUtils.isEmpty(vipInfo.getCpid())) {
			throw new BussinessException("9054", "cpid不能为空");
		}
		if (StringUtils.isEmpty(vipInfo.getSpid())) {
			throw new BussinessException("9054", "spid不能为空");
		}
		if (StringUtils.isEmpty(vipInfo.getOrderid())) {
			throw new BussinessException("9054", "orderid不能为空");
		}
		if (StringUtils.isEmpty(vipInfo.getChannel())) {
			throw new BussinessException("9054", "channel不能为空");
		}
		String sign = vipInfo.getSignature();
		if (StringUtils.isEmpty(sign)) {
			throw new BussinessException("9054", "signature不能为空");
		}
		CpSpBean cpSpBean = CpSpDao.getCpSp("twowei","10011");
		String pwd = cpSpBean.getPwd();
		
		String data = vipInfo.getUserid() + vipInfo.getActivetype()+vipInfo.getOrderid()+ pwd + vipInfo.getTimestamp() + vipInfo.getChannel();
		String localSign = MD5.MD5Encode(data);
		System.out.println(">>>>>>>>>>>>>>>>用户上传的sign:"+sign);
		System.out.println(">>>>>>>>>>>>>>>>服务生成的sign:"+localSign);
		if (!localSign.equals(sign)) {
			throw new BussinessException("9054", "signature校验失败");
		}
		vipInfo.setPwd(pwd);
		
	}
	/**
	 * Sign校验
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	private void checkSignGet(HxVipInfo vipInfo) throws BussinessException {
		if (StringUtils.isEmpty(vipInfo.getUserid())) {
			throw new BussinessException("9054", "userid不能为空");
		}
		if (StringUtils.isEmpty(vipInfo.getCpid())) {
			throw new BussinessException("9054", "cpid不能为空");
		}
		if (StringUtils.isEmpty(vipInfo.getSpid())) {
			throw new BussinessException("9054", "spid不能为空");
		}
		if (StringUtils.isEmpty(vipInfo.getOrderid())) {
			throw new BussinessException("9054", "orderid不能为空");
		}
		if (StringUtils.isEmpty(vipInfo.getChannel())) {
			throw new BussinessException("9054", "channel不能为空");
		}
		if (StringUtils.isEmpty(vipInfo.getUsertype())) {
			throw new BussinessException("9054", "usertype不能为空");
		}
		if (StringUtils.isEmpty(vipInfo.getUserval())) {
			throw new BussinessException("9054", "userval不能为空");
		}
		String sign = vipInfo.getSignature();
		if (StringUtils.isEmpty(sign)) {
			throw new BussinessException("9054", "signature不能为空");
		}
		CpSpBean cpSpBean = CpSpDao.getCpSp("twowei","10011");
		String pwd = cpSpBean.getPwd();
		
		String data = vipInfo.getUserid() + vipInfo.getActivetype()+vipInfo.getOrderid()+ pwd + vipInfo.getTimestamp() + vipInfo.getChannel()+vipInfo.getUsertype() + vipInfo.getUserval();
		String localSign = MD5.MD5Encode(data);
		System.out.println(">>>>>>>>>>>>>>>>用户上传的sign:"+sign);
		System.out.println(">>>>>>>>>>>>>>>>服务生成的sign:"+localSign);
		if (!localSign.equals(sign)) {
			throw new BussinessException("9054", "signature校验失败");
		}
		vipInfo.setPwd(pwd);
		
		HxProduct goods = hxDao.getProdcutBean(vipInfo.getCpid(), vipInfo.getSpid());
		vipInfo.setVipid(goods.getVipid());
		vipInfo.setVipchannel(goods.getVipchannel());
		vipInfo.setVippwd(goods.getVippwd());
		
	}
	
	/**
	 * userid校验与解密
	 * 
	 * @param activityInfo
	 * @throws BussinessException
	 */
	private void checkMob(HxVipInfo vipInfo) throws BussinessException {
		String userid = vipInfo.getUserid();
		if(StringUtils.isEmpty(userid)) {
			throw new BussinessException("9054", "userid不能为空");
		}
		try {
			userid = DesUtil.decode(userid, vipInfo.getPwd());
			if(!UsermobUtil.isValid(userid)) {
				throw new BussinessException("9055", "userid不正确");
			}
		}catch(Exception e) {
			throw new BussinessException("9056", "userid加密错误");
		}
		vipInfo.setUserid(userid);
	}
	
	/**
	 * userval校验与解密
	 * @param vipInfo
	 * @throws BussinessException
	 */
	private void checkUserval(HxVipInfo vipInfo) throws BussinessException {
		String userval = vipInfo.getUserval();
		if(StringUtils.isEmpty(userval)) {
			throw new BussinessException("9054", "userval不能为空");
		}
		try {
			userval= DesUtil.decode(userval, vipInfo.getPwd());
			if("4".equals(vipInfo.getUsertype()) && !UsermobUtil.isValid(userval)) {
				throw new BussinessException("9055", "userval不正确");
			}
		}catch(Exception e) {
			throw new BussinessException("9056", "userval加密错误");
		}
		vipInfo.setUserval(userval);
	}
	
	
	/**
	 * 验证后向产品，是否可以领取会员
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	private void checkProduct(HxVipInfo vipInfo) throws BussinessException {
		HxProduct goods = hxDao.getProdcutBean(vipInfo.getCpid(), vipInfo.getSpid());
		if(goods == null ){
			throw new BussinessException("9083", "该活动相应后向产品已失效");
		}
		if(!"1".equals(goods.getBusitype()) && !"3".equals(goods.getBusitype())){
			throw new BussinessException("9083", "该活动相应后向产品不赠送会员");
		}
		vipInfo.setVipid(goods.getVipid());
		vipInfo.setVipchannel(goods.getVipchannel());
		vipInfo.setVippwd(goods.getVippwd());
	}
	
	/**
	 * 验证订单号已经领取会员
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkOrderidQuery(HxVipInfo vipInfo) throws BussinessException {
		if(StringUtils.isEmpty(vipInfo.getOrderid())) {
			throw new BussinessException("9054", "orderid不能为空");
		}
		
		String [] ids = vipInfo.getOrderid().split("_");
		if(ids.length<=0){
			throw new BussinessException("9054", "orderid异常");
		}
		if(ids.length>10){
			throw new BussinessException("9054", "orderid量太大");
		}
	}
	/**
	 * 验证订单号已经领取会员
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkOrderidGet(HxVipInfo vipInfo) throws BussinessException {
		if(StringUtils.isEmpty(vipInfo.getOrderid())) {
			throw new BussinessException("9054", "orderid不能为空");
		}
		int num;
		try {
			num=NumberUtils.toInt(vipDao.getRequestId(vipInfo.getVipid(),vipInfo.getOrderid()));
		}catch(Exception e) {
			throw e;
		}
		if(num > 0) {
			throw new BussinessException("9057", "同一订单(orderid)不能重复领取会员");
		}
	}

	
	/**
	 * 请求校验
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkVipQuery(HxVipInfo vipInfo) throws BussinessException {
		comonService.checkTimestamp(vipInfo.getTimestamp());
		this.checkActivity(vipInfo);
		this.checkSignQuery(vipInfo);
		this.checkMob(vipInfo);
		this.checkProduct(vipInfo);
		this.checkOrderidQuery(vipInfo);
	}
	
	public void checkVipGet(HxVipInfo vipInfo) throws BussinessException {
		this.checkRedis(vipInfo);
		comonService.checkTimestamp(vipInfo.getTimestamp());
		this.checkActivity(vipInfo);
		this.checkSignGet(vipInfo);
		this.checkMob(vipInfo);
		this.checkUserval(vipInfo);
		this.checkProduct(vipInfo);
		this.checkOrderidGet(vipInfo);
	}
	
	
	/**
	 * 会员领取资格
	 * 
	 * @param info
	 * @throws BussinessException
	 */
//	public void queryVip(HxVipInfo info) throws BussinessException {
//		try {
//			HxOrder bean = hxDao.getOrderBean(info.getUserid(), info.getOrderid(), info.getCpid(), info.getSpid());
//			if(bean == null ) {
//				throw new BussinessException("3005", "此订单不存在");
//			}
//			
//			if(!"1".equals(bean.getVipstatus())){
//				if("0".equals(bean.getVipstatus())){
//					throw new BussinessException("3006", "此订单会员已经领取");
//				}
//				if("3".equals(bean.getVipstatus())){
//					throw new BussinessException("3006", "此订单会员领取失败");
//				}
//				if("4".equals(bean.getVipstatus())){
//					throw new BussinessException("3006", "此订单没有会员领取");
//				}
//				throw new BussinessException("3006", "此订单会员领取状态异常");
//			}
//			
//			if(bean.getVipendtime()==null || bean.getVipendtime().getTime()<=System.currentTimeMillis()){
//				throw new BussinessException("3006", "此订单会员领取已经过期");
//			}
//			
//		} catch (Exception e) {
//			e.printStackTrace();
//			
//			if (e instanceof BussinessException)
//				throw (BussinessException) e;
//			else
//				throw new BussinessException("3007", "此订单查询异常");
//		}
//	}
	
	public List<Map<String,String>> queryVipList(HxVipInfo info) throws BussinessException {
		List<Map<String,String>> result = new ArrayList<Map<String, String>>();
		String [] ids = info.getOrderid().split("_");
		for(String orderid:ids){
			Map<String,String> map = new HashMap<String,String>();
			map.put("orderid", orderid);
			try {
				HxOrder bean = hxDao.getOrderTwBean(info.getUserid(), orderid, info.getCpid(), info.getSpid());
				if(bean == null ) {
					throw new BussinessException("3005", "此订单不存在");
				}
				
				if(!"1".equals(bean.getVipstatus())){
					if("0".equals(bean.getVipstatus())){
						map.put("usertype", bean.getVipusertype()==null?"":bean.getVipusertype());
						map.put("userval", bean.getVipuserval()==null?"":bean.getVipuserval());
						map.put("viptime", bean.getVipgettime()==null?"":new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(bean.getVipgettime()));
						throw new BussinessException("0000", "此订单会员已经领取");
					}
					if("3".equals(bean.getVipstatus())){
						throw new BussinessException("3154", "此订单会员领取失败");
					}
					if("4".equals(bean.getVipstatus())){
						throw new BussinessException("3153", "此订单没有会员领取");
					}
					if("8".equals(bean.getVipstatus())){
						throw new BussinessException("3152", "此订单会员领取已冻结");
					}
					throw new BussinessException("3155", "此订单会员领取状态异常");
				}
				
				if(bean.getVipendtime()==null || bean.getVipendtime().getTime()<=System.currentTimeMillis()){
					throw new BussinessException("3151", "此订单会员领取已经过期");
				}
				
				map.put("vipcode", "0");
				map.put("vipinfo", "");
			} catch (Exception e) {
				e.printStackTrace();
				
				if (e instanceof BussinessException){
					BussinessException ex = (BussinessException)e;
					map.put("vipcode", ex.getCode());
					map.put("vipinfo", ex.getMessage());
				}else{
					map.put("vipcode", "3007");
					map.put("vipinfo", "此订单查询异常");
				}
			}
			result.add(map);
		}
		
		return result;
	}
	
	
	/**
	 * 会员领取
	 * 
	 * @param info
	 * @throws BussinessException
	 */
	public void giveVip(HxVipInfo info) throws BussinessException {
		try {
			HxOrder bean = hxDao.getOrderTwBean(info.getUserid(), info.getOrderid(), info.getCpid(), info.getSpid());
			if(bean == null ) {
				throw new BussinessException("3005", "此订单不存在");
			}
			
			if(!"1".equals(bean.getVipstatus())){
				if("0".equals(bean.getVipstatus())){
					throw new BussinessException("0000", "此订单会员已经领取");
				}
				if("3".equals(bean.getVipstatus())){
					throw new BussinessException("3154", "此订单会员领取失败");
				}
				if("4".equals(bean.getVipstatus())){
					throw new BussinessException("3153", "此订单没有会员领取");
				}
				if("8".equals(bean.getVipstatus())){
					throw new BussinessException("3152", "此订单会员领取已冻结");
				}
				throw new BussinessException("3155", "此订单会员领取状态异常");
			}
			
			if(bean.getVipendtime()==null || bean.getVipendtime().getTime()<=System.currentTimeMillis()){
				throw new BussinessException("3151", "此订单会员领取已经过期");
			}
			
		} catch (Exception e) {
			e.printStackTrace();
			
			if (e instanceof BussinessException)
				throw (BussinessException) e;
			else
				throw new BussinessException("3007", "此订单查询异常");
		}
		
		
		//调用直充接口送会员
		try {
			String remark = null;
			if("tencent".equals(info.getCpid())){
				remark = "会员产品";
			}
			
			String result = hxVipTwUtil.vipid(info.getUserid() ,info.getVipid(), info.getVipchannel(), info.getVippwd(), info.getUsertype(), info.getUserval(), info.getOrderid(), remark);
			JSONObject obj = JSON.parseObject(result);
			String resultcode = obj.getString("resultcode");
			String errorinfo = obj.getString("errorinfo");
			if(!"0".equals(resultcode) && !("10002".equals(resultcode) && "orderid不能重复使用".equals(errorinfo)) ){
				if(!"9070".equals(resultcode)){
					//会员失败
					hxDao.giveTwFail(info.getUserid(),info.getOrderid(), info.getCpid(), info.getSpid(),info.getUsertype(),info.getUserval());
				}
				throw new BussinessException(resultcode, errorinfo);
			}
		} catch (Exception e) {
			e.printStackTrace();
			if (e instanceof BussinessException)
				throw (BussinessException) e;
			else
				throw new BussinessException("8000", e.getMessage());
		}
		
		//会员成功
		hxDao.giveTwSuccess(info.getUserid(),info.getOrderid(), info.getCpid(), info.getSpid(),info.getUsertype(),info.getUserval());
	}
	
	
	
}
