package com.chinacreator.video.service.vip;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import com.chinacreator.video.bean.CpSpBean;
import com.chinacreator.video.bean.YoukuHX;
import com.chinacreator.video.bean.YoukuHXBean;
import com.chinacreator.video.exception.BussinessException;
import com.chinacreator.video.mapper.CpSpDao;
import com.chinacreator.video.mapper.YoukuXHDao;
import com.chinacreator.video.redis.RedisService;
import com.chinacreator.video.service.ComonService;
import com.chinacreator.video.util.AESUtil;
import com.chinacreator.video.util.DesUtil;
import com.chinacreator.video.util.MD5;
import com.chinacreator.video.util.UsermobUtil;

@Component
public class VipHXService {
	
	private static final String MOB_KEY = "ac22273abb2f4960";
	
	@Autowired
	private RedisService redisService;
	
	@Autowired
	private CpSpDao CpSpDao;
	
	@Autowired
	private YoukuXHDao youkuHXDao;

	@Autowired
	private ComonService comonService;

	@Autowired
	private YoukuHXVipService youkuVipService;
	
	
	/**
	 * 并发控制
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkRedis(YoukuHX vipInfo) throws BussinessException {
		long count = redisService.incr("hxvip_" + vipInfo.getCpid() + "_" + vipInfo.getSpid() + "_" + vipInfo.getUserid() + "_" + vipInfo.getOrderid());
		if (count > 1) {
			throw new BussinessException("8001", "并发请求，请稍后再试");
		}
	}
	
	/**
	 * Sign校验
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkSign(YoukuHX vipInfo) throws BussinessException {
		if (StringUtils.isEmpty(vipInfo.getCpid())) {
			throw new BussinessException("9054", "cpid不能为空");
		}
		if (StringUtils.isEmpty(vipInfo.getSpid())) {
			throw new BussinessException("9054", "spid不能为空");
		}
		if (StringUtils.isEmpty(vipInfo.getOrderid())) {
			throw new BussinessException("9054", "orderid不能为空");
		}
		if (StringUtils.isEmpty(vipInfo.getUserid())) {
			throw new BussinessException("9054", "userid不能为空");
		}
		String sign = vipInfo.getSignature();
		if (StringUtils.isEmpty(sign)) {
			throw new BussinessException("9054", "signature不能为空");
		}
		CpSpBean cpSpBean = CpSpDao.getCpSp(vipInfo.getCpid(), vipInfo.getSpid());
		String pwd = cpSpBean.getPwd();
		String data = vipInfo.getOrderid() + vipInfo.getUserid() + vipInfo.getTimestamp()+ pwd;
		String localSign = MD5.MD5Encode(data);
		System.out.println(">>>>>>>>>>>>>>>>用户上传的sign:"+sign);
		System.out.println(">>>>>>>>>>>>>>>>服务生成的sign:"+localSign);
		if (!localSign.equals(sign)) {
			throw new BussinessException("9054", "signature校验失败");
		}
		
		vipInfo.setPwd(pwd);
		System.out.println(">>>>>>>>>>>>>>>>密码:"+vipInfo.getPwd());
	}
	
	/**
	 * userid校验与解密
	 * 
	 * @param activityInfo
	 * @throws BussinessException
	 */
	public void checkMob(YoukuHX activityInfo) throws BussinessException {
		String userid = activityInfo.getUserid();
		if(StringUtils.isEmpty(userid)) {
			throw new BussinessException("9054", "userid不能为空");
		}
		try {
			userid = DesUtil.decode(userid, activityInfo.getPwd());
			if(userid.length() > 11) {
				userid = AESUtil.decrypt(userid, MOB_KEY);
			}
			if(!UsermobUtil.isValid(userid)) {
				throw new BussinessException("9055", "userid不正确");
			}
		}catch(Exception e) {
			throw new BussinessException("9056", "userid加密错误");
		}
		activityInfo.setUserid(userid);
	}
	
	
	/**
	 * 请求校验
	 * 
	 * @param vipInfo
	 * @throws BussinessException
	 */
	public void checkVip(YoukuHX vipInfo) throws BussinessException {
		this.checkRedis(vipInfo);
		comonService.checkTimestamp(vipInfo.getTimestamp());
		this.checkSign(vipInfo);
		this.checkMob(vipInfo);
	}
	
	
	
	/**
	 * 会员领取
	 * 
	 * @param info
	 * @throws BussinessException
	 */
	public void giveVip(YoukuHX info) throws BussinessException {
		try {

			YoukuHXBean bean = youkuHXDao.findOrderid(info.getUserid(), info.getCpid(), info.getSpid(), info.getOrderid());
			if(bean == null ) {
				throw new BussinessException("3005", "此优酷订单不存在");
			}
			
			if(!"1".equals(bean.getVipstatus())){
				throw new BussinessException("3006", "此优酷订单已经领取过会员");
			}
			
		} catch (Exception e) {
			e.printStackTrace();
			
			throw new BussinessException("3007", "查询优酷订单异常");
		}
		
		
		System.out.println(">>>>>>>>>>>>>>>>开始送优酷会员");
		youkuVipService.giveVip(info);
		
		//会员成功
		youkuHXDao.giveSuccess(info.getUserid(), info.getCpid(), info.getSpid(), info.getOrderid());
	}
	
	
	
}
